/**
 *
 * VkApp.C
 * 
 * This file contains the implementation for the VkApp
 * class.
 *
 * Chris Toshok
 * Copyright (C) 1994
 * The Hungry Programmers, Inc
 *
 **/

#ifndef lint
static char rcsid[] = "$Id: VkApp.C,v 1.8 1995/04/10 01:51:00 toshok Exp $";
#endif /* lint */

#include <Vk/VkApp.h>
#include <Vk/builtinbusy.h>
#include <Vk/VkInfoDialog.h>

#include <X11/cursorfont.h>
#include <assert.h>
#include <string.h>
#include <iostream.h>
#include <stdlib.h>

VkApp *theApplication = NULL;

String VkApp::_defaultResources[] = {
//"-*iconic:      False",
"-*debugResources:             False",
"-*busyCursorForeground:       White",
"-*busyCursorBackground:       Black",
"-*normalCursorForeGround:     White",
"-*normalCursorBackground:     Black",
"-*debugXtMode:                False",
"-*silenceWarnings:            False",
NULL,
};

const char *
VkApp::ViewKitReleaseString = "ViewKit Release 1.0";

const int
VkApp::ViewKitMinorRelease = 0;

const int 
VkApp::ViewKitMajorRelease = 1;

XtResource VkApp::_resourceList[] = {
    { "iconic", "Iconic", XmRBoolean, sizeof (Boolean),
      XtOffset (VkApp *, _startupIconified), XmRImmediate, (XtPointer)False},
    { "debugResources", "DebugResources", XmRBoolean, sizeof (Boolean),
      XtOffset (VkApp *, _dumpResourceFile), XmRImmediate, (XtPointer)False},
};

VkApp::VkApp(char *appClassName,
	     int *argc,
	     char **argv,
	     XrmOptionDescRec *options,
	     int numOptions): VkComponent (appClassName)
{
    _currentCursor = None;
    _mainWindow = NULL;
    theApplication = this;
    _appClassName = strdup(appClassName);

    _applicationArgc = *argc;
    _applicationArgv = (char**)new char*[_applicationArgc];

    int i;
    for (i=0; i<_applicationArgc; i++)
	_applicationArgv[i] = strdup(argv[i]);

    _busyDepth = 0;

    _busyDialog = theBusyDialog;
    _cursorType = NotAnimated;

    _startupIconified = False;

    XtToolkitInitialize();
  
    _theApp = XtCreateApplicationContext();

    _theDisplay = XtOpenDisplay(_theApp,
				NULL,
				appClassName,
				appClassName,
				options,
				numOptions,
				argc,
				argv);

    if (!_theDisplay){
	cerr << "VkApp::VkApp() : Couldn't open display <" << XDisplayName(NULL)<< ">, bombing." << endl;
	exit(1);
    }

    _busyCursor = XCreateFontCursor(_theDisplay, XC_watch);

    char *slash = strrchr(argv[0], (int)'/');

    Arg args[1];
    
    if (slash == NULL)
	slash = argv[0];
    else
	slash++;

    XtSetArg(args[0], XtNmappedWhenManaged, False);

    _baseWidget = XtAppCreateShell(slash, appClassName,
				   applicationShellWidgetClass,
				   _theDisplay,
				   args, 1);

    _aboutDialog = theInfoDialog;

    setDefaultResources(baseWidget(),_defaultResources);
}

int
VkApp::parseCommandLine(XrmOptionDescRec *options,
                        Cardinal numOptions)
{
    XrmDatabase line_db = NULL;
    XrmDatabase display_db = XrmGetStringDatabase(XResourceManagerString(_theDisplay));

    XrmParseCommand(&line_db, options, numOptions,
		    _name,
		    &_applicationArgc, _applicationArgv);

    XrmCombineDatabase(display_db, &line_db, True);
    XrmSetDatabase(_theDisplay, line_db);

    getResources(_resourceList, XtNumber(_resourceList));

    return _applicationArgc;
}

void
VkApp::handleRawEvent(XEvent *event)
{
    int i;

    for (i=_winList.size()-1; i>=0; i--){

	assert (_winList[i] != NULL);

	((VkSimpleWindow*)_winList[i])->handleRawEvent(event);
    }
}

void
VkApp::handlePendingEvents()
{
    XEvent ev;

    while (XtAppPending(_theApp)){
	XtAppNextEvent(_theApp, &ev);
	if(!XtDispatchEvent(&ev))
	    handleRawEvent(&ev);
    }
}

void
VkApp::run()
{
    XEvent event;

//    if (_dumpResourceFile)
	XrmPutFileDatabase(XrmGetDatabase (_theDisplay), "/usr/tmp/resources.db");

    if (_currentCursor != None)
	showCursor(_currentCursor);

    for (;;) { /* we needn't return */
	XtAppNextEvent(_theApp, &event);
	if(!XtDispatchEvent(&event))
	    handleRawEvent(&event);
    }
}

const char *
VkApp::className()
{
    return _appClassName;
}

Widget
VkApp::baseWidget()
{
    return _baseWidget;
}

char *
VkApp::argv(int index) const
{
    return _applicationArgv[index];
}

char **
VkApp::argv() const
{
    return _applicationArgv;
}

int
VkApp::argc() const
{
    return _applicationArgc;
}

void
VkApp::terminate(int status)
{
    exit(status);
}

char *
VkApp::applicationClassName() const
{
    return _appClassName;
}

VkSimpleWindow *
VkApp::mainWindow() const
{
    return _mainWindow;
}

void
VkApp::setMainWindow(VkSimpleWindow *window)
{
    _mainWindow = window;
}

void
VkApp::setVersionString(const char *versionInfo)
{
    _versionString = strdup(versionInfo);
}

const char *
VkApp::versionString()
{
    return _versionString;
}

void
VkApp::setAboutDialog(VkDialogManager *dialog)
{
    _aboutDialog = dialog;
}

void
VkApp::setBusyDialog(VkBusyDialog *newDialog)
{
    if (newDialog)
	_busyDialog = newDialog;
    else
	_busyDialog = theBusyDialog;
}

void
VkApp::busy(char *msg,
	    VkSimpleWindow *window)
{
    _busyDepth++;

    showCursor(busyCursor());

    if (msg != NULL)
	if (window == NULL)
	    _busyDialog->post(msg);
	else 
	{
	    _busyDialog->post(msg, NULL, NULL, NULL, NULL, NULL, window);
	}

    handlePendingEvents();
}

void
VkApp::progressing(const char *msg)
{
    if (_cursorType == Animated)
	_busyCursorList->progress();

    if (msg != NULL)
	_busyDialog->post(msg);
}

void
VkApp::notBusy()
{
    if (_busyDepth){ // We were busy to begin with
	_busyDepth--;
    }

    if (_busyDepth == 0){
	setNormalCursor(None);
	_busyDialog->unpostAll();
    }
}

void
VkApp::setBusyCursor(VkCursorList *animatedCursor)
{
    _busyCursorList = animatedCursor;
}
 
void
VkApp::setBusyCursor(Cursor c)
{
    _busyCursor = c;
}

Cursor
VkApp::busyCursor()
{
    if (_cursorType == Animated)
	return (*_busyCursorList)[_busyCursorList->index()];
    return _busyCursor;
}

void
VkApp::setNormalCursor(Cursor c)
{
    _currentCursor = c;
    showCursor(None);
}

Cursor
VkApp::normalCursor()
{
    return _currentCursor;
}

void
VkApp::showCursor(Cursor c)
{
    int i;
 
    if (c == None)
	for (i = 0; i < _winList.size(); i++){

	    assert (_winList[i] != NULL);

	    if (XtIsRealized(((VkSimpleWindow*)_winList[i])->mainWindowWidget()))
		XDefineCursor(_theDisplay, XtWindow(((VkSimpleWindow*)_winList[i])->mainWindowWidget()), _currentCursor);
	}
    else
	for (i = 0; i < _winList.size(); i++){

	    assert (_winList[i] != NULL);

	    if (XtIsRealized(((VkSimpleWindow*)_winList[i])->mainWindowWidget()))
		XDefineCursor(_theDisplay, XtWindow(((VkSimpleWindow*)_winList[i])->mainWindowWidget()), c);
	}
    handlePendingEvents();
}
 
void
VkApp::show()
{
    int i;

    for (i=0; i< _winList.size(); i++){
 
	assert (_winList[i] != NULL);

	_winList[i]->show();
    }
}

void
VkApp::hide()
{
    int i;

    for (i=0; i< _winList.size(); i++){

	assert (_winList[i] != NULL);

	_winList[i]->hide();
    }
}

void
VkApp::iconify()
{
    int i;

    for (i=0; i< _winList.size(); i++){
  
	assert (_winList[i] != NULL);

	((VkSimpleWindow*)_winList[i])->iconify();
    }
}

void
VkApp::open()
{
    int i;

    for (i=0; i< _winList.size(); i++){

	assert (_winList[i] != NULL);
 
	((VkSimpleWindow*)_winList[i])->open();
    }
}

void
VkApp::raise()
{
    int i;

    for (i=0; i< _winList.size(); i++){

	assert (_winList[i] != NULL);

	((VkSimpleWindow*)_winList[i])->raise();
    }
}

void
VkApp::lower()
{
    int i;

    for (i=0; i< _winList.size(); i++){

	assert (_winList[i] != NULL);

	((VkSimpleWindow*)_winList[i])->lower();
    }
}

void
VkApp::startupIconified(const Boolean flag)
{
    _startupIconified = flag;
}

Boolean
VkApp::startupIconified() const
{
    return _startupIconified;
}

XtAppContext
VkApp::appContext() const
{
    return _theApp;
}

Display *
VkApp::display() const
{
    return _theDisplay;
}

char *
VkApp::name() const
{
    char *slash = strrchr(argv(0), (int)'/');

    if (slash == NULL)
	return argv(0);
    else
	return (++slash);
}

char*
VkApp::shellGeometry() const
{
    return NULL; // For Now
}

void
VkApp::quitYourself()
{
    assert (_mainWindow != NULL);

    int i;
    int all_deleted = TRUE;

    for (i=_winList.size()-1 ; i >= 0; i--){
	if (_winList[i] == _mainWindow)
	    continue;
	if (_winList[i]->okToQuit()){
 
	    assert (_winList[i] != NULL);

	    delete _winList[i];

	    _winList.remove(_winList[i]);
	}
	else
	    all_deleted = FALSE;
    }
    if (_mainWindow->okToQuit()){
	delete _mainWindow;
	_winList.remove(_mainWindow);
    }
    else
	all_deleted = FALSE;
  
    if (all_deleted)
	terminate();
}

void
VkApp::registerWindow(VkSimpleWindow *win)
{
    assert(win != NULL);

    if (_mainWindow == NULL)
	setMainWindow(win);
    _winList.add(win);
}

void
VkApp::unregisterWindow(VkSimpleWindow *win)
{
    assert(win != NULL);

    int mainWindowDeleted = False;

    if (_mainWindow == win){
	mainWindowDeleted = True;
        printf ("Main window deleted,\n whatever shall we do?\n");
    }

    if (win->okToQuit()){
	_winList.remove(win);

	if (_winList.size() == 0){
            printf ("no other windows, exitting...\n");
	    terminate();
        }
    }

    if (mainWindowDeleted){
        printf ("choosing another one\n");
	_mainWindow = (VkSimpleWindow*)_winList[0];
    }
}
