/**
 *
 * VkCallbackList.C
 *
 * This file contains the class implementation for
 * the VkCallbackList object, which makes the implementation
 * of VkCallbackObject substantially simpler.
 *
 * Chris Toshok
 * Copyright (C) 1994, 1995
 * The Hungry Programmers, Inc.
 *
 **/

#include <Vk/VkCallbackList.h>
#include <string.h>
#include <stdlib.h>

VkCallbackList::VkCallbackList(const char *name)
{
    _name = strdup(name);
    _numberOfCallbacks = 0;
    _maxNumberOfCallbacks = 10;

    callbacks = (VkCallbackListEntry*)calloc(_maxNumberOfCallbacks, sizeof(VkCallbackListEntry));
}

void
VkCallbackList::addMemberFunction(VkCallbackObject *component,
				  VkCallbackMethod callbackMember,
				  void *clientData)
{
    callbacks[_numberOfCallbacks].type = VKCALLBACKMETHOD;

    callbacks[_numberOfCallbacks].component = component;
    callbacks[_numberOfCallbacks].callbackMemberFunction = callbackMember;
    callbacks[_numberOfCallbacks].clientData = clientData;

    _numberOfCallbacks++;

    if (_numberOfCallbacks == _maxNumberOfCallbacks){
	_maxNumberOfCallbacks *= 2;
	callbacks = (VkCallbackListEntry*)realloc(callbacks, _maxNumberOfCallbacks * 
						  sizeof(VkCallbackListEntry));
    }
}

void
VkCallbackList::addCStyleFunction(VkCallbackFunction callbackFunction,
				  void *clientData)
{
    callbacks[_numberOfCallbacks].type = VKCALLBACKFUNCTION;

    callbacks[_numberOfCallbacks].callbackFunction = callbackFunction;
    callbacks[_numberOfCallbacks].clientData = clientData;

    _numberOfCallbacks++;


    if (_numberOfCallbacks == _maxNumberOfCallbacks){
	_maxNumberOfCallbacks *= 2;
	callbacks = (VkCallbackListEntry*)realloc(callbacks, _maxNumberOfCallbacks * 
						  sizeof(VkCallbackListEntry));
    }
}

void
VkCallbackList::removeMemberFunction(VkCallbackObject *otherObject,
				     VkCallbackMethod memberFunction,
				     void *clientData)
{
    int i,j;

    for (i=0; i<_numberOfCallbacks; i++)
	if (callbacks[i].type == VKCALLBACKMETHOD){
	    if (callbacks[i].component == otherObject
		&& callbacks[i].callbackMemberFunction == memberFunction
		&& callbacks[i].clientData == clientData){

		for (j=i; j<_numberOfCallbacks; j++)
		    callbacks[j] = callbacks[j+1];

		_numberOfCallbacks--;
	    }
	}
}

void
VkCallbackList::removeCStyleFunction(VkCallbackFunction callbackFunction,
				     void *clientData)
{
    int i,j;

    for (i=0; i<_numberOfCallbacks; i++)
	if (callbacks[i].type == VKCALLBACKFUNCTION){
	    if (callbacks[i].callbackFunction == callbackFunction
		&& callbacks[i].clientData == clientData){

		for (j=i; j<_numberOfCallbacks; j++)
		    callbacks[j] = callbacks[j+1];

		_numberOfCallbacks--;
	    }
	}
}

void
VkCallbackList::removeAll()
{
    free(callbacks);

    _maxNumberOfCallbacks = 10;

    callbacks = (VkCallbackListEntry*)malloc(sizeof(VkCallbackListEntry*)*_maxNumberOfCallbacks);

    _numberOfCallbacks = 0;
}

void
VkCallbackList::removeAllMethods(VkCallbackObject *obj)
{
    int i,j;

    for (i=0; i<_numberOfCallbacks; i++)
	if (callbacks[i].type == VKCALLBACKMETHOD
	    && callbacks[i].component == obj){

	    for (j=i ; j < _numberOfCallbacks; j++)
		callbacks[j] = callbacks[j+1];

	    _numberOfCallbacks--;
	}
}

void
VkCallbackList::call(VkCallbackObject *caller, void *callData)
{
    int i;

    for (i=0; i< _numberOfCallbacks ; i++)
	switch(callbacks[i].type){
	case VKCALLBACKFUNCTION:
	    (*(callbacks[i].callbackFunction))
		(caller,
		 callbacks[i].clientData,
		 callData);
	    break;
	case VKCALLBACKMETHOD:
	    ((callbacks[i].component)->*
	     (callbacks[i].callbackMemberFunction))
		(caller,
		 callbacks[i].clientData,
		 callData);
	    break;
	}
}
