/**
 * 
 * VkCallbackObject.C
 * This file defines the class implementation for the
 * VkCallbackObject class, which forms the base class
 * for all of the user interface components in the Open
 * ViewKit library.
 *
 * Chris Toshok
 * Copyright (C) 1994
 * The Hungry Programmer, Inc
 *
 **/

#ifndef lint
static char rcsid[]="$Id: VkCallbackObject.C,v 1.2 1995/01/23 10:07:15 toshok Exp $";
#endif /* lint */

#include <Vk/VkCallbackObject.h>
#include <Vk/VkCallbackList.h>

#include <stdlib.h>
#include <string.h>
#include <iostream.h>

VkCallbackObject::VkCallbackObject()
{
    numberOfCallbackLists=0;
    maxCallbackLists = 10;

    callbackLists = (VkCallbackList**)malloc(maxCallbackLists * sizeof(VkCallbackList*));
}

void
VkCallbackObject::addNewCallbackList(const char *name)
{
    if (numberOfCallbackLists == maxCallbackLists-1){
	maxCallbackLists *= 2;
	callbackLists = (VkCallbackList **)realloc(callbackLists, 
						   maxCallbackLists * sizeof(VkCallbackList*));
    }

    callbackLists[numberOfCallbackLists] = new VkCallbackList(name);
    
    numberOfCallbackLists++;
}

void
VkCallbackObject::addCallback(const char *name,
			      VkCallbackObject *component,
			      VkCallbackMethod callbackFunction,
			      void *clientData)
{
    int i;
    
    // First we check to see that a callback of this type hasn't been registered before.
    // If it has, we add it to its list.  Otherwise, we create a new callback list for it.
    
    for (i=0; i<numberOfCallbackLists; i++){
	if (!strcmp(name, callbackLists[i]->name())){ // we found it
	    callbackLists[i]->addMemberFunction(component, callbackFunction, clientData);
	    return;
	}
    }
    
    addNewCallbackList(name);
    addCallback(name, component, callbackFunction, clientData);
}

void
VkCallbackObject::removeCallback(const char *name,
				 VkCallbackObject *otherObject,
				 VkCallbackMethod memberFunction,
				 void *clientData)
{
    int i;

    // First we need to find the callback list named NAME, then we need to
    // locate the object and method within the callback list.  We then
    // pop everything down one.

    for (i=0; i<numberOfCallbackLists; i++){
	if (!strcmp(name, callbackLists[i]->name())){ // we found it
	    callbackLists[i]->removeMemberFunction(otherObject, memberFunction, clientData);
	    return;
	}
    }
}

void
VkCallbackObject::addCallback(const char *name,
			      VkCallbackFunction callbackFunction,
			      void *clientData)
{
    int i;
    int found = 0;
    // First we check to see that a callback of this type hasn't been registered before.
    // If it has, we add it to its list.  Otherwise, we create a new callback list for it.

    for (i=0; i<numberOfCallbackLists; i++)
	if (!strcmp(name, callbackLists[i]->name())){ // we found it
	    callbackLists[i]->addCStyleFunction(callbackFunction, clientData);
	    return;
	}
    addNewCallbackList(name);

    addCallback(name, callbackFunction, clientData);
}

void
VkCallbackObject::removeCallback(const char *name,
				 VkCallbackFunction callbackFunction,
				 void *clientData)
{
    int i;

    for (i=0; i<numberOfCallbackLists; i++){
	if (!strcmp(name, callbackLists[i]->name())){ // we found it
	    callbackLists[i]->removeCStyleFunction(callbackFunction, clientData);
	    return;
	}
    }
}

void
VkCallbackObject::removeAllCallbacks(void)
{
    int i;
    for (i=0; i< numberOfCallbackLists; i++){
	callbackLists[i]->removeAll();
    }
}    


void
VkCallbackObject::removeAllCallbacks(VkCallbackObject *obj)
{
    int i;
    
    for (i=0; i<numberOfCallbackLists; i++)
	callbackLists[i]->removeAllMethods(obj);
}

void
VkCallbackObject::callCallbacks(const char *const callback,
				void *callData)
{
    int i;

    for (i=0; i<numberOfCallbackLists; i++)
	if (!strcmp(callbackLists[i]->name(), callback))
	    callbackLists[i]->call(this, callData);
}
