/**
 *
 * VkCheckBox.C
 *
 * This file contains the class implementation for the
 * VkCheckBox class, which allows programmers to
 * create groups of check-box type toggle buttons
 * with a title and frame surrounding them.
 *
 * Copyright (C) 1994
 * The Hungry Programmers, Inc.
 * All Rights Reserved
 *
 **/

#include <Vk/VkCheckBox.h>
#include <Xm/LabelG.h>
#include <Xm/ToggleB.h>
#include <Xm/Frame.h>
#include <Xm/RowColumn.h>

VkCheckBox::VkCheckBox(const char *name, 
                       Widget parent) : VkComponent(name)
{
    _baseWidget = XmCreateFrame(parent, (char*)name, NULL, 0);

    installDestroyHandler();

    _label = XtVaCreateManagedWidget("label",
				     xmLabelGadgetClass, _baseWidget,
				     XmNchildType, XmFRAME_TITLE_CHILD,
				     XmNchildVerticalAlignment, XmALIGNMENT_CENTER,
				     NULL);

    _rc = XtVaCreateManagedWidget("rc",
				  xmRowColumnWidgetClass, _baseWidget,
				  XmNchildType, XmFRAME_WORKAREA_CHILD,
				  NULL);

    _widgetList = new VkWidgetList();
}

Widget 
VkCheckBox::addItem(char *name, 
                    Boolean state,
                    XtCallbackProc proc,
                    XtPointer clientDate)
{
    Widget newItem;  
  
    newItem = XtVaCreateManagedWidget(name,
				      xmToggleButtonWidgetClass, _rc,
				      NULL);

    XmToggleButtonSetState(newItem, state, False);

    if (proc)
        XtAddCallback (newItem, XmNvalueChangedCallback, valueChangedCallback, this);

    _widgetList->add(newItem);

    XtAddCallback (newItem, XmNvalueChangedCallback, valueChangedCallback, this);
    return newItem;
}

void 
VkCheckBox::setValue(int index, 
                     Boolean newValue)
{
    if (index <= _widgetList->size())
        XmToggleButtonSetState((*_widgetList)[index],
                               newValue, False);
}

void 
VkCheckBox::setValues(Boolean *values, 
                      int numValues)
{
    int i;

    if (numValues <= _widgetList->size())
	for (i=0; i < numValues; i++)
	    XmToggleButtonSetState((*_widgetList)[i],
				   values[i], False);
}

int 
VkCheckBox::getValue(int i)
{
    if (i <= _widgetList->size())
	return XmToggleButtonGetState((*_widgetList)[i]);
    else
	return -1;
}

void 
VkCheckBox::valueChanged(int index, 
                         Boolean)
{
    callCallbacks(VkCheckBox::itemChangedCallback,
		  (void*)index);
}

void
VkCheckBox::valueChangedCallback(Widget w, 
                                 XtPointer clientData,
                                 XtPointer callData)
{
    int index;
    VkCheckBox *obj = (VkCheckBox *)clientData;
    XmToggleButtonCallbackStruct *cb = (XmToggleButtonCallbackStruct *)callData;

    index = obj->_widgetList->indexOfItem(w);

    if (index != -1)
	obj->valueChanged(index, cb->set);
}

const char *
VkCheckBox::className()
{
    return "VkCheckBox";
}

const char *const
VkCheckBox::itemChangedCallback = "VkCheckBox::itemChangedCallback";
