/**
 *
 * VkComponent.C
 * 
 * This file contains the class implementation of
 * the VkComponent Class.  For all intents and purposes,
 * this class is useless to the end-user, as it is almost
 * completely abstract.
 *
 * Chris Toshok
 * Copyright (C) 1994
 * The Hungry Programmer, Inc
 *
 **/

#ifndef lint
static char rcsid[]="$Id: VkComponent.C,v 1.2 1995/01/23 10:07:17 toshok Exp $";
#endif /* lint */

#include <Vk/VkComponent.h>
#include <Vk/VkApp.h>
#include <iostream.h>
#include <stdlib.h>

const char *const VkComponent::deleteCallback = "VkComponentDeleteCallback";

const char *
VkComponent::name() const
{
    return _name;
}

Widget
VkComponent::baseWidget()
{
    return _baseWidget;
}

VkComponent::VkComponent(const char *name)
{
    if (name)
	_name = strdup(name);
    else
	_name = NULL;
    _baseWidget = NULL;
}

VkComponent::~VkComponent()
{
    callCallbacks(VkComponent::deleteCallback,
		  NULL);

    removeDestroyHandler();

    XtDestroyWidget(_baseWidget);

    free(_name);

    _name = NULL;
}


VkComponent::operator Widget() const
{
    return _baseWidget;
}

void
VkComponent::show()
{
    /* By default, just manage the baseWidget */
    XtManageChild (_baseWidget);
}

void 
VkComponent::hide()
{
    /* By default, just unmanage the baseWidget */
    XtUnmanageChild(_baseWidget);
}

void
VkComponent::installDestroyHandler()
{
    XtAddCallback(_baseWidget, XmNdestroyCallback, widgetDestroyedCallback, (XtPointer)this);
}

void
VkComponent::removeDestroyHandler()
{
    XtRemoveCallback(_baseWidget, XmNdestroyCallback, widgetDestroyedCallback, (XtPointer)this);
}

void 
VkComponent::widgetDestroyed()
{
    _baseWidget = NULL;
}

void
VkComponent::setDefaultResources(const Widget parent,
				 const String *resourceSpec)
{
    int current=0;
    Display *display = XtDisplay(parent);

    XrmDatabase display_db = XrmGetDatabase(display);

    xrm_db = XrmGetStringDatabase("");
  
    for (current = 0;
	 resourceSpec[current] != NULL;
	 current ++){
	if (resourceSpec[current][0] == '-'){
	    char temp[1000];
	    sprintf (temp, "%s*%s\n", theApplication->className(), &(resourceSpec[current][1]));

	    XrmPutLineResource (&xrm_db, (const char *)temp);
	}
	else{
	    char temp[1000];
	    sprintf (temp, "%s*%s%s\n", theApplication->className(), (char*)_name, resourceSpec[current]);

	    XrmPutLineResource (&xrm_db, temp);
	}
    
    }

    XrmCombineDatabase(xrm_db, &display_db, False);
    
    XrmSetDatabase(display, display_db);
}

void
VkComponent::getResources(const XtResourceList resources,
 			  const int numResources)
{
    if (_baseWidget == NULL){
	fprintf (stderr, "_baseWidget NULL inside of getResources\n");
	exit(1);
    }
    
    XtGetApplicationResources(_baseWidget,
			      this,
			      resources,
			      numResources,
			      NULL,0);
}

void
VkComponent::widgetDestroyedCallback(Widget,
				     XtPointer clientData,
				     XtPointer)
{
    ((VkComponent*)clientData)->widgetDestroyed();
}
