/**
 *
 * VkDialogManager.C
 *
 * This file implements the base, abstract class
 * from which all other VkDialogs are derived.
 *
 * Chris Toshok
 * Copyright (C) 1994
 * The Hungry Programmers, Inc
 *
 **/

#ifndef lint
static char rcsid[] = "$Id: VkDialogManager.C,v 1.10 1995/04/10 02:05:44 toshok Exp $";
#endif /* lint */

#include <Vk/VkDialogManager.h>
#include <Vk/VkApp.h>
#include <Vk/VkFormat.h>

#include <Xm/AtomMgr.h>
#include <Xm/Protocols.h>
#include <stdlib.h>
#include <iostream.h>
#include <stdio.h>

const char *
VkDialogManager::className()
{
    return "VkDialogManager";
}

VkDialogManager::VkDialogManager(const char *name) : VkComponent(name)
{
    _okButton = False;
    _cancelButton = False;
    _applyButton = False;
    _helpButton = False;

    _centered = False;

    _allowMultipleDialogs = True;

    number_of_dialogs_cached = 0;
    max_number_of_dialogs_cached = 10;

    dialogs = (Widget*)malloc(max_number_of_dialogs_cached * sizeof (Widget));
    parents = (Widget*)malloc(max_number_of_dialogs_cached * sizeof (Widget));
  
    _okLabel = 
       _cancelLabel =
       _applyLabel = NULL;
}

Widget
VkDialogManager::GetChildButton(Widget w,
				unsigned int child)
{
    if (XtIsSubclass(w, xmSelectionBoxWidgetClass))
	switch (child){
	case XmDIALOG_OK_BUTTON:
	case XmDIALOG_APPLY_BUTTON:
	case XmDIALOG_CANCEL_BUTTON:
	case XmDIALOG_HELP_BUTTON:
	case XmDIALOG_DEFAULT_BUTTON:
	case XmDIALOG_LIST:
	case XmDIALOG_LIST_LABEL:
	case XmDIALOG_SELECTION_LABEL:
	case XmDIALOG_TEXT:
	case XmDIALOG_WORK_AREA:
	case XmDIALOG_SEPARATOR:
	    return XmSelectionBoxGetChild(w, child);
	    break;
	default:
	    return (Widget)NULL;
	    break;
	}
    else/* assuming that anything else is a message box */
	switch (child){
	case XmDIALOG_OK_BUTTON:
	case XmDIALOG_CANCEL_BUTTON:
	case XmDIALOG_HELP_BUTTON:
	case XmDIALOG_DEFAULT_BUTTON:
	case XmDIALOG_MESSAGE_LABEL:
	case XmDIALOG_SEPARATOR:
	case XmDIALOG_SYMBOL_LABEL:
	    return XmMessageBoxGetChild(w, child);
	    break;
	default:
	    return (Widget)NULL;
	}
}

int 
VkDialogManager::_cachedDialogFor(Widget parent)
{
    int i;

    for (i=0; i< number_of_dialogs_cached; i++)
	if (parent == parents[i] && !XtIsManaged(dialogs[i]))
	    return 1;
    
    return 0;
}

Widget
VkDialogManager::_getDialog(Widget parent)
{
    int i;

    for (i=0; i < number_of_dialogs_cached; i++)
	if (parent == parents[i] && !XtIsManaged(dialogs[i]))
	    return dialogs[i];
}

void
VkDialogManager::_addDialog(Widget parent, Widget dialog)
{
    parents[number_of_dialogs_cached] = parent;
    dialogs[number_of_dialogs_cached] = dialog;

    number_of_dialogs_cached++;

    if (number_of_dialogs_cached == max_number_of_dialogs_cached){
	max_number_of_dialogs_cached *= 2;
	parents = (Widget*)realloc(parents, max_number_of_dialogs_cached * sizeof (Widget));
	dialogs = (Widget*)realloc(dialogs, max_number_of_dialogs_cached * sizeof (Widget));
    }
}

Widget
VkDialogManager::prepost(const char *msg,
			 XtCallbackProc okCB,
			 XtCallbackProc cancelCB,
			 XtCallbackProc applyCB,
			 XtPointer clientData,
			 const char *helpString,
			 VkSimpleWindow *parent)
{
    Arg args[10];
    int n;
    Widget w; /* used when managing/unmanaging individual buttons */
    Widget box;
    XmString xmstr;

    if (parent)
    {
	if (_cachedDialogFor(parent->baseWidget()))
	    box = _getDialog(parent->baseWidget());
	else
	{
	    box = createDialog(parent->baseWidget());
	    _addDialog(parent->baseWidget(), box);
	}
    }
    else 
    {
	printf ("Creating Dialog as child of application shell \n");
	box = createDialog(theApplication->baseWidget());
	_addDialog(theApplication->baseWidget(), box);
    }

    _baseWidget = XtParent(box); // the dialog shell
    
    w = GetChildButton(box, XmDIALOG_OK_BUTTON);
    if (w){
	if (_okButton){
	    XtManageChild(w);
	    if (okCB)
		XtAddCallback(box, XmNokCallback, okCB, clientData);
	}
	else{
	    if (XtIsManaged(w))
		XtUnmanageChild(w);
	}
	XtAddCallback(box, XmNokCallback, _dialogBoxOK, this);	
    }
    if (_okLabel)
    {
	printf ("Setting OK label\n");
        xmstr = XmStringCreateSimple(_okLabel); 
        XtVaSetValues(box, XmNokLabelString, xmstr, NULL);
        free(_okLabel);
        _okLabel = NULL;
	XmStringFree(xmstr);
    }
    
    w = GetChildButton(box, XmDIALOG_CANCEL_BUTTON);
    if (w){
	if (_cancelButton){
	    XtManageChild(w);
	    if (cancelCB)
		XtAddCallback(box, XmNcancelCallback, cancelCB, clientData);	
	}
	else{
	    if (XtIsManaged(w))
		XtUnmanageChild(w);
	}
	XtAddCallback(box, XmNcancelCallback, _dialogBoxCANCEL, this);	
    }
    if (_cancelLabel)
    {
	printf ("Setting cancel label\n");
        xmstr = XmStringCreateSimple(_cancelLabel); 
        XtVaSetValues(box, XmNcancelLabelString, xmstr, NULL);
	free(_cancelLabel);
	_cancelLabel = NULL;
	XmStringFree(xmstr);
    }
    
    w = GetChildButton(box, XmDIALOG_APPLY_BUTTON);
    if (w){
	if (_applyButton){
	    XtManageChild(w);
	    if (applyCB)
		XtAddCallback(box, XmNapplyCallback, applyCB, clientData);	
	}
	else{
	    if (XtIsManaged(w))
		XtUnmanageChild(w);
	}  
	XtAddCallback(box, XmNapplyCallback, _dialogBoxAPPLY, this);	
    }
    if (_applyLabel)
    {
	printf ("Setting apply label\n");
        xmstr = XmStringCreateSimple(_applyLabel); 
        XtVaSetValues(box, XmNapplyLabelString, xmstr, NULL);
	free(_applyLabel);
        _applyLabel = NULL;
	XmStringFree(xmstr);
    }
    
    w = GetChildButton(box, XmDIALOG_HELP_BUTTON);
    if (w)
	if (_helpButton != True || helpString == NULL){
	    if (XtIsManaged(w))
		XtUnmanageChild(w);
	    
	}

    if (_dialogShellTitle){
	char *title = (char*)VkGetResource(_baseWidget,
					   _dialogShellTitle,
					   _dialogShellTitle,
					   XmRString,
					   _dialogShellTitle);
	
	XtSetArg(args[0], XtNtitle, title);
	
	XtSetValues(_baseWidget, args, 1);
	
    }
    n = 0;
    if (msg){
	XmString xmstr;
	char *mmsg = (char*)VkGetResource(_baseWidget,
					  msg,
					  msg,
					  XmRString,
					  msg);
	xmstr = XmStringCreateLtoR(mmsg,
				   XmFONTLIST_DEFAULT_TAG);
	if (xmstr){
	    XtSetArg(args[n],XmNmessageString, xmstr); n++;
	    XtSetArg(args[n],XmNselectionLabelString, xmstr); n++;
        }
    }
    
    XtSetValues(box, args, n);

    n=0;

    return box;
}

Widget
VkDialogManager::post(const char *msg,
		      XtCallbackProc okCB,
		      XtCallbackProc cancelCB,
		      XtCallbackProc applyCB,
		      XtPointer clientData,
		      const char *helpString,
		      VkSimpleWindow *parent,
		      Widget *parentWidget)
{

    Widget dialogBox = prepost(msg,
			       okCB,
			       cancelCB,
			       applyCB,
			       clientData,
			       helpString,
			       parent);
    
    /*... there's nothing more to do but manage and return it :)*/

    _baseWidget = XtParent(dialogBox);

    XtVaSetValues(_baseWidget, 
		  XtNmappedWhenManaged, False,
		  NULL);

    XtManageChild(dialogBox);

    setupWindowManagerProperties(dialogBox);

    if (_centered){// don't check the parent stuff yet  FIX ME
	int width = DisplayWidth(theApplication->display(), 0);
	int height = DisplayHeight(theApplication->display(), 0);

	int dialog_width;
	int dialog_height;

	XtVaGetValues(_baseWidget, 
		      XtNheight, &dialog_height, 
		      XtNwidth, &dialog_width,
		      NULL);

	XtVaSetValues(_baseWidget,
                      XmNgeometry, VkFormat("%dx%d", (int)((width -dialog_width	)/2), (int)((height - dialog_height)/2)),
                      NULL);

    }

    XtVaSetValues(_baseWidget,
		  XtNmappedWhenManaged, True,
		  NULL);

    XmUpdateDisplay(_baseWidget);

/*    theApplication->handlePendingEvents();*/

    return dialogBox;
}


Widget
VkDialogManager::postModal(const char *msg,
			   XtCallbackProc okCB,
			   XtCallbackProc cancelCB,
			   XtCallbackProc applyCB,
			   XtPointer clientData,
			   const char *helpString,
			   VkSimpleWindow *parent,
			   Widget *parentWidget)
{
    Arg args[1];

    Widget dialogBox = prepost(msg,
			       okCB,
			       cancelCB,
			       applyCB,
			       clientData,
			       helpString,
			       parent);
    
    /*... we set the dialogStyle and manage/return it */
    
    XtSetArg(args[0], XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL);
    XtSetValues(dialogBox, args, 1);

    _baseWidget = XtParent(dialogBox);

    XtVaSetValues(_baseWidget, 
		  XtNmappedWhenManaged, False,
		  NULL);

    XtManageChild(dialogBox);

    setupWindowManagerProperties(dialogBox);

    if (_centered){// don't check the parent stuff yet  FIX ME
	int width = DisplayWidth(theApplication->display(), 0);
	int height = DisplayHeight(theApplication->display(), 0);

	int dialog_width;
	int dialog_height;

	XtVaGetValues(_baseWidget, 
		      XtNheight, &dialog_height, 
		      XtNwidth, &dialog_width,
		      NULL);

	XtVaSetValues(_baseWidget,
                      XmNgeometry, VkFormat("%dx%d", (int)((width -dialog_width	)/2), (int)((height - dialog_height)/2)),
                      NULL);
    }

    XtVaSetValues(_baseWidget,
		  XtNmappedWhenManaged, True,
		  NULL);

    XmUpdateDisplay(_baseWidget);

/*    theApplication->handlePendingEvents();*/
    return dialogBox;
}

Widget
VkDialogManager::postBlocked(const char *msg,
			     XtCallbackProc okCB,
			     XtCallbackProc cancelCB,
			     XtCallbackProc applyCB,
			     XtPointer clientData,
			     const char *helpString,
			     VkSimpleWindow *parent,
			     Widget *parentWidget)
{
    Arg args[1];

    Widget dialogBox = prepost(msg,
			       okCB,
			       cancelCB,
			       applyCB,
			       clientData,
			       helpString,
			       parent);
    
    /*... we set up our own event loop, and don't return
      until the user presses one of the buttons.  When
      we do return, the value is the widget id. The
      dialog box is again DIALOG_FULL_APPLICATION_MODAL */

    
    XtSetArg(args[0], XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL);
    XtSetValues(dialogBox, args, 1);
    
    done_with_event_loop = False;

    _baseWidget = XtParent(dialogBox);

    XtVaSetValues(_baseWidget, 
		  XtNmappedWhenManaged, False,
		  NULL);

    XtManageChild(dialogBox);

    setupWindowManagerProperties(dialogBox);

    if (_centered){// don't check the parent stuff yet  FIX ME
	int width = DisplayWidth(theApplication->display(), 0);
	int height = DisplayHeight(theApplication->display(), 0);

	int dialog_width;
	int dialog_height;

	XtVaGetValues(_baseWidget, 
		      XtNheight, &dialog_height, 
		      XtNwidth, &dialog_width,
		      NULL);
	
	XtVaSetValues(_baseWidget,
                      XmNgeometry, VkFormat("%dx%d", (int)((width -dialog_width	)/2), (int)((height - dialog_height)/2)),
                      NULL);
    }

    XtVaSetValues(_baseWidget,
		  XtNmappedWhenManaged, True,
		  NULL);
    
    while (!done_with_event_loop)
	XtAppProcessEvent(theApplication->appContext(), XtIMAll);

    XmUpdateDisplay(_baseWidget);

    /*theApplication->handlePendingEvents();*/
    return(dialogBox);
}

Widget
VkDialogManager::postBlocked(const char *msg,
                             VkSimpleWindow *above)
{
    return postBlocked(msg, NULL, NULL, NULL, NULL, NULL, above);
}

VkDialogManager::VkDialogReason
VkDialogManager::postAndWait(const char *msg,
			     Boolean ok, 
			     Boolean cancel,
			     Boolean apply,
			     const char *helpString,
			     VkSimpleWindow *parent,
			     Widget *parentWidget)
{
    Arg args[1];

    Widget dialogBox = prepost(msg,
			       NULL,
			       NULL,
			       NULL,
			       NULL,
			       helpString,
			       parent);

    /*... we set up our own event loop, and don't return
      until the user presses one of the buttons.  When
      we do return, the value is of type VkDialogReason. 
      The dialog box is again DIALOG_FULL_APPLICATION_MODAL */

    
    XtSetArg(args[0], XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL);
    XtSetValues(dialogBox, args, 1);
    
    done_with_event_loop = False;

    _baseWidget = XtParent(dialogBox);

    XtVaSetValues(_baseWidget, 
		  XtNmappedWhenManaged, False,
		  NULL);

    XtManageChild(dialogBox);

    setupWindowManagerProperties(dialogBox);

    if (_centered){// don't check the parent stuff yet  FIX ME
	int width = DisplayWidth(theApplication->display(), 0);
	int height = DisplayHeight(theApplication->display(), 0);

	int dialog_width;
	int dialog_height;

	XtVaGetValues(_baseWidget, 
		      XtNheight, &dialog_height, 
		      XtNwidth, &dialog_width,
		      NULL);
	
	XtVaSetValues(_baseWidget,
                      XmNgeometry, VkFormat("%dx%d", (int)((width -dialog_width	)/2), (int)((height - dialog_height)/2)),
                      NULL);
    }

    XtVaSetValues(_baseWidget,
		  XtNmappedWhenManaged, True,
		  NULL);
    
    while (!done_with_event_loop)
	XtAppProcessEvent(theApplication->appContext(), XtIMAll);

    return _action;
}

void
VkDialogManager::centerOnScreen(Boolean flag)
{
    _centered = flag;
}

void
VkDialogManager::setTitle(const char *nextTitle)
{
    if (_dialogShellTitle)
	delete _dialogShellTitle;

    _dialogShellTitle = strdup((char*)nextTitle);
}

void 
VkDialogManager::setButtonLabels(const char *okLabel,
                                 const char *cancelLabel,
                                 const char *applyLabel)
{
    if (okLabel)
       _okLabel = strdup(okLabel);

    if (cancelLabel)
       _cancelLabel = strdup(cancelLabel);

    if (applyLabel)
       _applyLabel = strdup(applyLabel);
}


Widget
VkDialogManager::lastPosted()
{
    int i;

    for (i=number_of_dialogs_cached-1; i>=0; i--)
	if (XtIsManaged(dialogs[i]))
	    return dialogs[i];

    return NULL;
}

void
VkDialogManager::unpost()
{
    printf ("There are %d dialogs currently cached\n", number_of_dialogs_cached);

    if (lastPosted())
	XtUnmanageChild(lastPosted());
}

void
VkDialogManager::unpost(Widget w)
{
    int i;

    for (i=0; i<number_of_dialogs_cached; i++)
	if (dialogs[i] == w){
	    XtUnmanageChild(dialogs[i]);
	    break;
	}
}

void
VkDialogManager::unpostAll()
{
    int i;

    for (i=0; i<number_of_dialogs_cached; i++)
	XtUnmanageChild(dialogs[i]);
}

void
VkDialogManager::_dialogBoxOK(Widget dialog, 
			      XtPointer clientData, 
			      XtPointer callData)
{
    VkDialogManager *obj = (VkDialogManager*)clientData;

    /* if we were in our own event loop, quit it */
    obj->_action = VkDialogManager::OK;
    obj->done_with_event_loop = True;

    obj->ok(dialog, callData);
}

void
VkDialogManager::_dialogBoxCANCEL(Widget dialog, 
				  XtPointer clientData, 
				  XtPointer callData)
{
    VkDialogManager *obj = (VkDialogManager*)clientData;
    /* if we were in our own event loop, quit it */
    obj->_action = VkDialogManager::CANCEL;
    obj->done_with_event_loop = True;

    obj->cancel(dialog, callData);
}

void
VkDialogManager::_dialogBoxAPPLY(Widget dialog, 
				 XtPointer clientData, 
				 XtPointer callData)
{
    VkDialogManager *obj = (VkDialogManager*)clientData;
    obj->apply(dialog, callData);
}

//
// Convenience wrappers for the post*() calls
//

Widget 
VkDialogManager::post (const char *msg,
		       VkSimpleWindow *parent)
{
    if (parent)
	printf ("parent is not null here\n");
    return post(msg, NULL, NULL, NULL, NULL, NULL, parent);
}

Widget 
VkDialogManager::post (const char *msg,
		       const char *helpString,
		       VkSimpleWindow *parent)
{
    return post(msg, NULL, NULL, NULL, NULL, helpString, parent);
}

Widget 
VkDialogManager::post (const char *msg,
		       XtCallbackProc okCB,
		       XtPointer clientData,
		       VkSimpleWindow *parent)
{
    return post(msg, okCB, NULL, NULL, clientData, NULL, parent);
}

Widget 
VkDialogManager::post (const char *msg,
		       XtCallbackProc okCB,
		       XtPointer clientData,
		       const char *helpString,
		       VkSimpleWindow *parent)
{
    return post(msg, okCB, NULL, NULL, clientData, helpString, parent);
}

Widget 
VkDialogManager::post (const char *msg,
		       XtCallbackProc okCB,
		       XtCallbackProc cancelCB,
		       XtPointer clientData,
		       char *helpString,
		       VkSimpleWindow *parent)
{
    return post(msg, okCB, cancelCB, NULL, clientData, helpString, parent);
}

Widget 
VkDialogManager::post (const char *msg,
		       XtCallbackProc okCB,
		       XtCallbackProc cancelCB,
		       XtCallbackProc applyCB,
		       XtPointer clientData,
		       VkSimpleWindow *parent)
{
    return post(msg, okCB, cancelCB, applyCB, clientData, NULL, parent);
}

Widget 
VkDialogManager::post (const char *msg,
		       Widget parent)
{
    return post(msg, NULL, NULL, NULL, NULL, NULL, NULL, &parent);
}

Widget 
VkDialogManager::post (const char *msg,
		       const char *helpString,
		       Widget parent)
{
    return post(msg, NULL, NULL, NULL, NULL, helpString, NULL, &parent);
}

Widget 
VkDialogManager::post (const char *msg,
		       XtCallbackProc okCB,
		       XtPointer clientData,
		       Widget parent)
{
    return post(msg, okCB, NULL, NULL, clientData, NULL, NULL, &parent);
}

Widget 
VkDialogManager::post (const char *msg,
		       XtCallbackProc okCB,
		       XtPointer clientData,
		       const char *helpString,
		       Widget parent)
{
    return post(msg, okCB, NULL, NULL, clientData, helpString, NULL, &parent);
}

Widget 
VkDialogManager::post (const char *msg,
		       XtCallbackProc okCB,
		       XtCallbackProc cancelCB,
		       XtPointer clientData,
		       Widget parent)
{
    return post(msg, okCB, cancelCB, NULL, clientData, NULL, NULL, &parent);
}

Widget 
VkDialogManager::post (const char *msg,
		       XtCallbackProc okCB,
		       XtCallbackProc cancelCB,
		       XtPointer clientData,
		       char *helpString,
		       Widget parent)
{
    return post(msg, okCB, cancelCB, NULL, clientData, helpString, NULL, &parent);
}

Widget 
VkDialogManager::post (const char *msg,
		       XtCallbackProc okCB,
		       XtCallbackProc cancelCB,
		       XtCallbackProc applyCB,
		       XtPointer clientData,
		       Widget parent)
{
    return post(msg, okCB, cancelCB, applyCB, clientData, NULL, NULL, &parent);
}


Widget 
VkDialogManager::postModal (const char *msg,
			    VkSimpleWindow *parent)
{
    return postModal(msg, NULL, NULL, NULL, NULL, NULL, parent);
}

Widget
VkDialogManager::postModal(const char *msg,
			   const char *helpString,
			   VkSimpleWindow *parent)
{
    return postModal(msg, NULL, NULL, NULL, NULL, helpString, parent);
}

Widget 
VkDialogManager::postModal(const char *msg,
			   XtCallbackProc okCB,
			   XtPointer clientData,
			   VkSimpleWindow *parent)
{
    return postModal(msg, okCB, NULL, NULL, clientData, NULL, parent);
}

Widget 
VkDialogManager::postModal(const char *msg,
			   XtCallbackProc okCB,
			   XtPointer clientData,
			   const char *helpString,
			   VkSimpleWindow *parent)
{
    return postModal(msg, okCB, NULL, NULL, clientData, helpString, parent);
}

Widget
VkDialogManager::postModal(const char *msg,
			   XtCallbackProc okCB,
			   XtCallbackProc cancelCB,
			   XtPointer clientData,
			   VkSimpleWindow *parent)
{
    return postModal(msg, okCB, cancelCB, NULL, clientData, NULL, parent);
}

Widget 
VkDialogManager::postModal(const char *msg,
			   XtCallbackProc okCB,
			   XtCallbackProc cancelCB,
			   XtPointer clientData,
			   char *helpString,
			   VkSimpleWindow *parent)
{
    return postModal(msg, okCB, cancelCB, NULL, clientData, helpString, parent);
}

Widget 
VkDialogManager::postModal(const char *msg,
			   XtCallbackProc okCB,
			   XtCallbackProc cancelCB,
			   XtCallbackProc applyCB,
			   XtPointer clientData,
			   VkSimpleWindow *parent)
{
    return postModal(msg, okCB, cancelCB, applyCB, clientData, NULL, parent);
}

Widget
VkDialogManager::postModal(const char *msg,
			   Widget parent)
{
    return postModal(msg, NULL, NULL, NULL, NULL, NULL, NULL, &parent);
}

Widget
VkDialogManager::postModal(const char *msg,
			   XtCallbackProc okCB,
			   XtPointer clientData,
			   Widget parent)
{
    return postModal(msg, okCB, NULL, NULL, clientData, NULL, NULL, &parent);
}

Widget 
VkDialogManager::postModal(const char *msg,
			   XtCallbackProc okCB,
			   XtPointer clientData,
			   const char *helpString,
			   Widget parent)
{
    return postModal(msg, okCB, NULL, NULL, clientData, helpString, NULL, &parent);
}

Widget 
VkDialogManager::postModal (const char *msg,
			    XtCallbackProc okCB,
			    XtCallbackProc cancelCB,
			    XtPointer clientData,
			    Widget parent)
{
    return postModal(msg, okCB, cancelCB, NULL, clientData, NULL, NULL, &parent);
}

Widget 
VkDialogManager::postModal (const char *msg,
			    XtCallbackProc okCB,
			    XtCallbackProc cancelCB,
			    XtPointer clientData,
			    char *helpString,
			    Widget parent)
{
    return postModal(msg, okCB, cancelCB, NULL, clientData, helpString, NULL, &parent);
}

Widget 
VkDialogManager::postModal (const char *msg,
			    XtCallbackProc okCB,
			    XtCallbackProc cancelCB,
			    XtCallbackProc applyCB,
			    XtPointer clientData,
			    Widget parent)
{
    return postModal(msg, okCB, cancelCB, applyCB, clientData, NULL, NULL, &parent);
}


VkDialogManager::VkDialogReason
VkDialogManager::postAndWait(const char *msg,
			     const char *helpString,
			     VkSimpleWindow *parent)
{
    return postAndWait(msg, TRUE, TRUE, FALSE, helpString, parent);
}

VkDialogManager::VkDialogReason
VkDialogManager::postAndWait(const char *msg,
			     VkSimpleWindow *parent)
{
    return postAndWait(msg, TRUE, TRUE, FALSE, NULL, parent);
}

void
VkDialogManager::setupWindowManagerProperties(Widget dialog)
{
    Widget shell = XtParent(dialog);

    Atom WM_DELETE_WINDOW;
    Atom WM_QUIT_APP;

    WM_DELETE_WINDOW = XmInternAtom(theApplication->display(),
				    "WM_DELETE_WINDOW", False);

    WM_QUIT_APP = XmInternAtom(theApplication->display(),
			       "_WM_QUIT_APP", True);
    
    XmAddWMProtocolCallback(shell, WM_DELETE_WINDOW, handle_wm, this);
}

void
VkDialogManager::handle_wm(Widget w, XtPointer clientData, XtPointer)
{
    VkDialogManager *obj = (VkDialogManager*)clientData;

    obj->unpost(w);
}
