/**
 *
 * VkFileSelectionDialog.C
 *
 * This file contains the class implementation for the
 * VkFileSelectionDialog class.
 *
 * Chris Toshok
 * Copyright (C) 1994
 * The Hungry Programmers, Inc
 *
 **/

#ifndef lint
static char rcsid[] = "$Id: VkFileSelectionDialog.C,v 1.5 1995/02/15 17:42:06 toshok Exp $";
#endif /* lint */


#include <Vk/VkFileSelectionDialog.h>
#include <Vk/VkApp.h>
#include <string.h>
#include <stdlib.h>
#include <iostream.h>

VkFileSelectionDialog::VkFileSelectionDialog(const char *name) : VkDialogManager(name)
{
    _okButton = True;
    _cancelButton = True;
    _applyButton = True;

    _dialogShellTitle = NULL;

    _directory = NULL;
    _pattern = NULL;
    _selection = NULL;
    _filename = NULL;
}

Widget
VkFileSelectionDialog::createDialog(Widget parent)
{
    Arg args[5];
    int n =0;
    XmString xmstr;

    XtSetArg(args[n], XmNdefaultButton, XmDIALOG_APPLY_BUTTON); n++;
    if (_directory){
	xmstr = XmStringCreateSimple(_directory);
	XtSetArg(args[n], XmNdirectory, xmstr); n++;
	XmStringFree(xmstr);
    }
    if (_pattern){
	xmstr = XmStringCreateSimple(_pattern);
	XtSetArg(args[n], XmNpattern, xmstr); n++;
	XmStringFree(xmstr);
    }
    if (_selection){
	xmstr = XmStringCreateSimple(_selection);
	XtSetArg(args[n], XmNdirSpec, xmstr);
	XmStringFree(xmstr);
    }

    Widget dialogBox = XmCreateFileSelectionDialog(parent,
						   (char*)_name,
						   args, n);

    XtAddCallback(dialogBox, XmNokCallback, _setFilenameCallback, this);

    return dialogBox;
}

void
VkFileSelectionDialog::setDirectory(const char *directory)
{
    Arg args[1];
    XmString xmstr;

    if (lastPosted() && XtIsManaged(lastPosted())){
	xmstr = XmStringCreateSimple((char*)directory);
	
	XtSetArg(args[0], XmNdirectory, xmstr);
	
	XtSetValues(lastPosted(), args, 1);
	XmStringFree(xmstr);
    }
    
    if (_directory)
	free(_directory);
    _directory = strdup(directory);
}

void
VkFileSelectionDialog::setFilterPattern(const char *pattern)
{
    Arg args[1];
    XmString xmstr;

    if (lastPosted() && XtIsManaged(lastPosted())){
	xmstr = XmStringCreateSimple((char*)pattern);
	
	XtSetArg(args[0], XmNpattern, xmstr);

	XtSetValues(lastPosted(), args, 1);
	XmFileSelectionDoSearch(lastPosted(), xmstr);
	XmStringFree(xmstr);
    }

    if (_pattern)
	free(_pattern);
    _pattern = strdup(pattern);
}

void
VkFileSelectionDialog::setSelection(const char *selection)
{
    Arg args[1];
    XmString xmstr;
    if (lastPosted() && XtIsManaged(lastPosted())){
	xmstr = XmStringCreateSimple((char*)selection);
	
	XtSetArg(args[0], XmNdirSpec, xmstr);
	
	XtSetValues(lastPosted(), args, 1);
	XmStringFree(xmstr);
    }
    if (_selection)
	free(_selection);
    _selection = strdup(selection);
}

const char *
VkFileSelectionDialog::fileName()
{
    return _filename;
}

void
VkFileSelectionDialog::setFilenameCallback(char *new_filename)
{
    _filename = strdup(new_filename);
}

void
VkFileSelectionDialog::_setFilenameCallback(Widget, XtPointer clientData, XtPointer callData)
{
    char *filename;

    XmFileSelectionBoxCallbackStruct *cb = (XmFileSelectionBoxCallbackStruct*)callData;
    VkFileSelectionDialog *obj = (VkFileSelectionDialog *)clientData;

    if (XmStringGetLtoR(cb->value, XmFONTLIST_DEFAULT_TAG, &filename)){
	obj->setFilenameCallback(filename);
	XtFree(filename);
    }
    else
	cerr << "This program is probably going to dump core now... " << endl;
}

const char *
VkFileSelectionDialog::className()
{
    return "VkFileSelectionDialog";
}

VkFileSelectionDialog *_theFileSelectionDialogManager = new VkFileSelectionDialog("FileSelectionDialogManager");
