/**
 *
 * VkFont.C
 *
 * This file contains the class implementation
 * VkFont, which encapsulates screen (as well
 * as printer) fonts, and provides an interface
 * to manipulating them.  The majority of operations,
 * however, should be performed with the VkFontManager
 * instead of individual fonts, so the application
 * maintains information of these changes.
 *
 * This class is roughly analogous to NSFont in
 * the OpenStep Specification, which inspired it.
 *
 * Copyright (C) 1994
 * The Hungry Programmers, Inc.
 * All Rights Reserved
 *
 **/

#include <Vk/VkFont.h>
#include <Vk/VkApp.h>
#include <Vk/VkFormat.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>

VkFont::VkFont (char *xlfd,
		char *fontName,
                float pointSize,
                VkFontTraitMask)
{
    FILE *fp;

    _pointSize = pointSize;

    _screenName = strdup(xlfd);

/*
    if ((fp = fopen(VkFormat("/usr/public/lib/ViewKit/AFM/%s", fontName), "r")) == NULL)
*/
	_hasAFM = False;
    /*else 
    {
	_hasAFM = parseAFM(fp);
	fclose(fp);
    }*/

    if (!_hasAFM)
        _fontName = strdup(fontName);

    fs = XLoadQueryFont(theApplication->display(),
			_screenName);
}

Boolean
VkFont::parseAFM(FILE *fp)
{
    char buf[1000];
    char first_field[1000];

    fgets(buf, 1000, fp);

    if (strncmp(buf, "StartFontMetrics", 16))
	return False;

    while (!feof(fp))
    {
	fscanf(fp, "%s ", first_field);

	if (!strcmp(first_field, "FamilyName"))
	{
	    fscanf(fp, "%s ", first_field);
	    _familyName = strdup(first_field);
	    continue;
	}
	else if (!strcmp(first_field, "FontName"))
	{
	    fscanf(fp, "%s ", first_field);
	    _fontName = strdup(first_field);
	    continue;
	}
	else 
	{
	    
	    fgets(buf, 1000, fp);
	    continue;
	}
    }
    return True;
}

Boolean 
VkFont::hasAFMDictionary()
{
    return _hasAFM;
}

const char *
VkFont::displayName()
{
    return _fontName;
}

const char *
VkFont::familyName()
{
  return NULL;
}

float 
VkFont::pointSize()
{
    return _pointSize;
}

VkFont *
VkFont::printerFont()
{
    return this;
}

XFontStruct *
VkFont::screenFont()
{
    return fs;
}

float 
VkFont::stringWidth(char *string)
{
    if (_hasAFM)
	return 0.0;/* do some nifty font metric things */
    else
	return (float)XTextWidth(fs, string, strlen(string));
}
  
float
VkFont::stringWidth(char *string, int num_chars)
{
    if (_hasAFM)
	return 0.0;/* do some nifty font metric things */
    else
	return (float)XTextWidth(fs, string, num_chars);
}

const float *
VkFont::widths()
{
  return NULL;
}

float 
VkFont::xAdvancementForGlyph(char aGlyph, 
			     char)
{
    if (_hasAFM)
	return 0.0;/* do some nifty thing with font metrics */
    else
	return XTextWidth(fs, &aGlyph, 1);
}

float 
VkFont::yAdvancementForGlyph(char)
{
    if (_hasAFM)
	return 0.0;/* do some nifty thing with font metrics */
    else
	return fs->ascent + fs->descent;
}

float 
VkFont::heightOfBoundingRect(char c)
{
    if (isspace (c))
	return 0;

    if (_hasAFM)
	return 0.0;/* do some nifty thing with font metrics */
    else
	return fs->ascent + fs->descent;
}

float 
VkFont::widthOfBoundingRect(char aGlyph)
{
    if (_hasAFM)
	return 0.0;/* do some nifty thing with font metrics */
    else
	return XTextWidth(fs, &aGlyph, 1);
}

float
VkFont::ascent()
{
    if (_hasAFM)
	return 0.0;/* do some nifty thing with the font metrics */
    else
	return fs->ascent;
}

float
VkFont::descent()
{
    if (_hasAFM)
	return 0.0;/* do some nifty thing with the font metrics */
    else
	return fs->descent;
}
