/**
 *
 * VkGangedGroup.C
 *
 * This file contains the class implementation
 * for the VkGangedGroup class, which allows the
 * programmer to keep a group of scrollbars or
 * scales at the same value, meaning if you update
 * one, they are all updated.
 *
 * Copyright (C) 1994
 * The Hungry Programmers, Inc.
 * All Rights Reserved
 *
 **/

#include <Vk/VkGangedGroup.h>
#include <Xm/ScrollBar.h>
#include <Xm/Scale.h>

VkGangedGroup::VkGangedGroup() : VkWidgetList()
{
}

void 
VkGangedGroup::add(Widget w)
{
  if (XmIsScrollBar(w)){
    XtAddCallback(w, XmNincrementCallback, _changeCallback, this);
    XtAddCallback(w, XmNdecrementCallback, _changeCallback, this);
    XtAddCallback(w, XmNpageIncrementCallback, _changeCallback, this);
    XtAddCallback(w, XmNpageDecrementCallback, _changeCallback, this);
    XtAddCallback(w, XmNtoTopCallback, _changeCallback, this);
    XtAddCallback(w, XmNtoBottomCallback, _changeCallback, this);
    XtAddCallback(w, XmNdragCallback, _changeCallback, this);
    XtAddCallback(w, XmNvalueChangedCallback, _changeCallback, this);
  }
  else if (XmIsScale(w)){
    XtAddCallback(w, XmNvalueChangedCallback, _changeCallback, this);
    XtAddCallback(w, XmNdragCallback, _changeCallback, this);
  }

  VkWidgetList::add(w);
}

void 
VkGangedGroup::add(VkComponent *obj)
{
  add(obj->baseWidget());
}

void 
VkGangedGroup::remove(Widget w)
{
  /* remove the callback.  see add() above. */

  VkWidgetList::remove(w);
}

void 
VkGangedGroup::remove(VkComponent *obj)
{
  remove (obj->baseWidget());
}

void 
VkGangedGroup::removeFirst()
{
  remove((*this)[0]);
}

void 
VkGangedGroup::removeLast()
{
  remove((*this)[size() -1]);
}

void
VkGangedGroup::_change(int value)
{
  int i;

  for (i=0; i < size(); i++)
      XtVaSetValues((*this)[i], XmNvalue, value, NULL);
}

void
VkGangedGroup::_changeCallback(Widget w,
                               XtPointer clientData,
                               XtPointer callData)
{
  VkGangedGroup *obj = (VkGangedGroup*)clientData;

  if (XmIsScrollBar(w)){
    XmScrollBarCallbackStruct *cb = (XmScrollBarCallbackStruct*)callData;
    
    obj->_change(cb->value);
  }
  else if (XmIsScale(w)){
    XmScaleCallbackStruct *cb = (XmScaleCallbackStruct*)callData;

    obj->_change(cb->value);
  }
}
