/**
 *
 * VkHelpPane.C
 *
 * This file contains the source code necessary for the implementation
 * of the VkHelpPane class.
 *
 * Chris Toshok
 * Copyright (C) 1994
 * The Hungry Programmers, Inc
 *
 **/

#ifndef lint
static char rcsid[] = "$Id: VkHelpPane.C,v 1.4 1995/04/10 01:48:32 toshok Exp $";
#endif /* lint */

#include <Vk/VkHelpPane.h>
/*#include <Vk/VkHelpAPI.h>*/
#include <Vk/VkFormat.h>
#include <iostream.h>
#include <X11/cursorfont.h>

const String 
VkHelpPane::defaultResourceSpec[] = {
  ".labelString: Help",
  ".mnemonic: H",
  "*helpOnContextMenuItem.labelString: Click for Help",
  "*helpOnContextMenuItem.mnemonic: C",
  "*helpOnContextMenuItem.accelerator: Shift<Key>F1",
  "*helpOnContextMenuItem.acceleratorText: Shift+F1",
  "*helpOverviewMenuItem.labelString: Overview",
  "*helpOverviewMenuItem.mnemonic: O",
  "*helpIndexMenuItem.labelString: Index",
  "*helpIndexMenuItem.mnemonic: I",
  "*helpKeysMenuItem.labelString: Keys & ShortCuts",
  "*helpKeysMenuItem.mnemonic: K",
  "*helpVersionMenuItem.labelString: Product Info",
  "*helpVersionMenuItem.mnemonic: P",
  NULL
};

VkHelpPane::VkHelpPane(const char *name,
		       VkMenuDesc *desc,
		       XtPointer defaultClientData) : VkSubMenu (name)
{
  VkMenuDesc *current;
  Cursor helpCursor;
  
  if (desc)
    for (current = desc;
	 current->menuType != END;
	 current ++)
      switch (current->menuType){
      case ACTION:
	addAction(current->name,
		  current->callback,
		  current->clientData==NULL ? defaultClientData : current->clientData);
	break;
      case CONFIRMFIRSTACTION:
	addConfirmFirstAction(current->name,
			      current->callback,
			      current->clientData==NULL ? defaultClientData : current->clientData);
	break;
      case TOGGLE:
	addToggle(current->name,
		  current->callback,
		  current->clientData==NULL ? defaultClientData : current->clientData);
	break;
      case LABEL:
	addLabel(current->name);
	break;
      case SEPARATOR:
	addSeparator();
	break;
      case SUBMENU:
	addSubmenu(current->name,
		   current->submenu,
		   defaultClientData);
	break;
      case RADIOSUBMENU:
	addRadioSubmenu(current->name,
			current->submenu,
			defaultClientData);
	break;
      }
  
  addAction("helpOnContextMenuItem", clickForHelpCallback, this);
  addAction("helpOverviewMenuItem", overviewCallback, this);
  addSeparator();
  addAction("helpIndexMenuItem", indexCallback, this);
  addAction("helpKeysMenuItem", keysCallback, this);
  addSeparator();
  addAction("helpVersionMenuItem", showVersionCallback, this);
}

VkHelpPane::VkHelpPane(Widget parent,
		       const char *name,
		       VkMenuDesc *desc,
		       XtPointer defaultClientData) : VkSubMenu (name)
{
  VkMenuDesc *current;
  Cursor helpCursor;
  
  if (desc)
    for (current = desc;
	 current->menuType != END;
	 current ++)
      switch (current->menuType){
      case ACTION:
	addAction(current->name,
		  current->callback,
		  current->clientData==NULL ? defaultClientData : current->clientData);
	break;
      case CONFIRMFIRSTACTION:
	addConfirmFirstAction(current->name,
			      current->callback,
			      current->clientData==NULL ? defaultClientData : current->clientData);
	break;
      case TOGGLE:
	addToggle(current->name,
		  current->callback,
		  current->clientData==NULL ? defaultClientData : current->clientData);
	break;
      case LABEL:
	addLabel(current->name);
	break;
      case SEPARATOR:
	addSeparator();
	break;
      case SUBMENU:
	addSubmenu(current->name,
		   current->submenu,
		   defaultClientData);
	break;
      case RADIOSUBMENU:
	addRadioSubmenu(current->name,
			current->submenu,
			defaultClientData);
	break;
      }


  addAction("helpOnContextMenuItem", clickForHelpCallback, this);
  addAction("helpOverviewMenuItem", overviewCallback, this);
  addSeparator();
  addAction("helpIndexMenuItem", indexCallback, this);
  addAction("helpKeysMenuItem", keysCallback, this);
  addSeparator();
  addAction("helpVersionMenuItem", showVersionCallback, this);
  
  _instantiate(parent);
}

VkHelpPane::~VkHelpPane()
{
  //empty
}

const char *
VkHelpPane::className()
{
  return "VkHelpPane";
}

void
VkHelpPane::_instantiate(Widget parent)
{
  Arg args[2];
  int i;

  setDefaultResources(parent,
		      defaultResourceSpec);
  
  helpCursor = XCreateFontCursor(theApplication->display(),
				 XC_question_arrow);

  if (_tearOff)
    XtSetArg(args[0], XmNtearOffModel, XmTEAR_OFF_ENABLED);
  else
    XtSetArg(args[0], XmNtearOffModel, XmTEAR_OFF_DISABLED);

  _pulldown = XmCreatePulldownMenu(parent,
				   (char *)_name,
				   args,1);

  XtSetArg(args[0], XmNsubMenuId, _pulldown);
  
  _baseWidget = XmCreateCascadeButtonGadget(parent,
					    (char *)_name,
					    args,1);

  XtManageChild(baseWidget());
  
  for (i=0; i<numberOfElements; i++)
    elements[i]->_instantiate(_pulldown);

  instantiated = TRUE;
}

void
VkHelpPane::showVersion()
{
    VkSimpleWindow *parent;

    theApplication->busy();

    if (theApplication->mainWindow())
	parent = theApplication->mainWindow();

    theApplication->aboutDialog()->setTitle(theApplication->name());

    if (theApplication->versionString())
	theApplication->aboutDialog()->post(theApplication->versionString(),
					    NULL,
					    NULL,
					    NULL,
					    NULL,
					    NULL,
					    parent);
    else
	theApplication->aboutDialog()->post(VkFormat("Application Name: <application>"),
					    NULL,
					    NULL,
					    NULL,
					    NULL,
					    NULL,
					    parent);
    
    theApplication->notBusy();
}

void
VkHelpPane::showVersionCallback(Widget, XtPointer clientData, XtPointer)
{
    VkHelpPane *obj = (VkHelpPane*)clientData;

    obj->showVersion();
}

static void
strprecat(char *a, char *b)
{
    char temp[1000];

    strcpy(temp,b);
    strcat(temp,a);
    strcpy(a,temp);
}

void
VkHelpPane::clickForHelp()
{
    XEvent ev;
    Widget helpWidget;
    char fullyQual[1000];

    helpWidget = XmTrackingEvent(theApplication->mainWindow()->baseWidget(),
				 helpCursor,
				 False,
				 &ev);

    if (helpWidget == NULL)
	return;

    Widget current = helpWidget;

    theApplication->busy();

    strcpy(fullyQual, XtName(current));
    
    do {
	strprecat(fullyQual, ".");
	current = XtParent(current);
	strprecat(fullyQual, XtName(current));
    } while (current != theApplication->baseWidget());

    theApplication->notBusy();

    //VkHelpMsg(fullyQual, NULL);
}

Widget
VkHelpPane::isParentAShell()
{
    Widget current = _baseWidget;

    theApplication->busy();

    while (!XtIsShell(current))
	current = XtParent(current);

    theApplication->notBusy();

    if (XtIsSubclass(current, applicationShellWidgetClass))
	return current;
    else
	return None;
}

void 
VkHelpPane::overview()
{
    Widget shell = isParentAShell();

    if (shell != None){
	//VkHelpMsg((char*)VkFormat("%s.overview",XtName(shell)), NULL);
    }
    else{
	//VkHelpMsg("overview", NULL);
    }
}

void
VkHelpPane::index()
{
    Widget shell = isParentAShell();
    
    if (shell != None){
	//VkHelpMsg((char*)VkFormat("%s.index",XtName(shell)), NULL);
    }
    else{
	//VkHelpMsg("index", NULL);
    }
}

void
VkHelpPane::keys()
{
    Widget shell = isParentAShell();
    
    if (shell != None){
	//VkHelpMsg((char*)VkFormat("%s.keys",XtName(shell)), NULL);
    }
    else{
	//VkHelpMsg("keys", NULL);
    }
}

void
VkHelpPane::clickForHelpCallback(Widget, XtPointer clientData, XtPointer)
{
    VkHelpPane *obj = (VkHelpPane*)clientData;

    obj->clickForHelp();
}

void
VkHelpPane::overviewCallback(Widget, XtPointer clientData, XtPointer)
{
    VkHelpPane *obj = (VkHelpPane*)clientData;

    obj->overview();    
}

void
VkHelpPane::indexCallback(Widget, XtPointer clientData, XtPointer)
{
    VkHelpPane *obj = (VkHelpPane*)clientData;

    obj->index();    
}

void
VkHelpPane::keysCallback(Widget, XtPointer clientData, XtPointer)
{
    VkHelpPane *obj = (VkHelpPane*)clientData;

    obj->keys();
}
