/**
 *
 * VkInput.C
 *
 * This file contains the implementation for the
 * VkInput class, which encapsulates callbacks
 * on file descriptors 
 *
 * Chris Toshok
 * Copyright (C) 1995
 * The Hungry Programmers, Inc.
 * All Rights Reserved
 *
 **/

#include <Vk/VkInput.h>
#include <Vk/VkApp.h>

VkInput::VkInput()
{
    _fd = -1;
}

VkInput::~VkInput()
{
    remove();
}

void
VkInput::attach(int fd, XtInputMask mask)
{
    if (_fd != -1)
	remove();

    _fd = fd;

    _id = XtAppAddInput(theApplication->appContext(),
			fd,
			(XtPointer)mask,
			inputHandler,
			(XtPointer)this);
}

void
VkInput::remove()
{
    _fd = -1;
    XtRemoveInput(_id);
}

int
VkInput::fd()
{
    return _fd;
}

int
VkInput::id()
{
    return _id;
}

void
VkInput::inputHandler(XtPointer clientData,
		      int *,
		      XtInputId *)
{
    VkInput *obj = (VkInput*)clientData;

    obj->callCallbacks(obj->inputCallback, obj);
}

const char *const
VkInput::inputCallback = "VkInput::inputCallback";
