/**
 *
 * VkMenu.C
 *
 * This file contains the source code for the implementation of the
 * VkMenu class, which forms the base class for the container classes
 * of menu elements.
 * 
 * Chris Toshok
 * Copyright (C) 1994
 * The Hungry Programmers, Inc
 *
 **/

#ifndef lint
static char rcsid[] = "$Id: VkMenu.C,v 1.2 1995/01/23 10:07:33 toshok Exp $";
#endif /* lint */

#include <Vk/VkMenu.h>
#include <Vk/VkSubMenu.h>
#include <Vk/VkMenuAction.h>
#include <Vk/VkMenuConfirmFirstAction.h>
#include <Vk/VkMenuToggle.h>
#include <Vk/VkMenuLabel.h>
#include <Vk/VkMenuSeparator.h>
#include <Vk/VkRadioSubMenu.h>
#include <Vk/VkOptionMenu.h>
#include <iostream.h>
#include <stdlib.h>

VkMenu::VkMenu(const char *name) : VkMenuItem ((char*)name)
{
    numberOfElements = 0;
    maxNumberOfElements = 10;
    _defaultClientData = NULL;
    elements = (VkMenuItem **)calloc(maxNumberOfElements, sizeof (VkMenuItem *));
}

VkMenuAction *
VkMenu::addAction(const char *name,
		  XtCallbackProc actionCallback,
		  XtPointer clientData,
		  int position)
{
    VkMenuAction *newAction = new VkMenuAction((char *)name);

    newAction->actionCallback = actionCallback;
    newAction->clientData = clientData;
  
    if (numberOfElements == maxNumberOfElements - 1){ // the array is currently full.
	maxNumberOfElements *= 2;
	elements = (VkMenuItem **)realloc(elements, maxNumberOfElements * sizeof (VkMenuItem *));
    }
  
    if (position == -1){ // place it at the end of the array
	elements[numberOfElements++] = newAction;
    }
    else{ // place it somewhere in the middle, and move all the others down
	int i;
    
	for (i=numberOfElements-1; i>=position; i--)
	    elements[i+1] = elements[i];
	elements[position] = newAction;
	numberOfElements++;
    }

    if (instantiated){
	switch(menuType()){
	case SUBMENU:
	    newAction->_instantiate(((VkSubMenu*)this)->pulldown());
	    break;
	case OPTION:
	    newAction->_instantiate(((VkOptionMenu*)this)->pulldown());
	    break;
	default:
	    newAction->_instantiate(baseWidget());
	    break;
	}
    }
    return newAction;
}

VkMenuAction *
VkMenu::addAction(const char */*name*/,
		  XtCallbackProc /*actionCallback*/,
		  XtCallbackProc /*undoCallback*/,
		  XtPointer /*clientData*/,
		  int /*position*/)
{
    return NULL;
}

VkMenuConfirmFirstAction *
VkMenu::addConfirmFirstAction(const char *name,
			      XtCallbackProc actionCallback,
			      XtPointer clientData,
			      int position)
{
    VkMenuConfirmFirstAction *newAction = new VkMenuConfirmFirstAction((char *)name);

    newAction->actionCallback = actionCallback;
    newAction->clientData = clientData;
  
    if (numberOfElements == maxNumberOfElements - 1){ // the array is currently full.
	maxNumberOfElements *= 2;
	elements = (VkMenuItem **)realloc(elements, maxNumberOfElements * sizeof (VkMenuItem *));
    }
  
    if (position == -1){ // place it at the end of the array
	elements[numberOfElements++] = newAction;
    }
    else{ // place it somewhere in the middle, and move all the others down
	int i;
    
	for (i=numberOfElements-1; i>=position; i--)
	    elements[i+1] = elements[i];
	elements[position] = newAction;
	numberOfElements++;
    }

    if (instantiated){
	switch(menuType()){
	case SUBMENU:
	    newAction->_instantiate(((VkSubMenu*)this)->pulldown());
	    break;
	case OPTION:
	    newAction->_instantiate(((VkOptionMenu*)this)->pulldown());
	    break;
	default:
	    newAction->_instantiate(baseWidget());
	    break;
	}
    }
    return newAction;
}

VkMenuToggle *
VkMenu::addToggle(const char *name,
		  XtCallbackProc actionCallback,
		  XtPointer clientData,
		  int state,
		  int position)
{
    VkMenuToggle *newToggle = new VkMenuToggle((char *)name);

    newToggle->actionCallback = actionCallback;
    newToggle->clientData = clientData;
  
    if (numberOfElements == maxNumberOfElements - 1){ // the array is currently full.
	maxNumberOfElements *= 2;
	elements = (VkMenuItem **)realloc(elements, maxNumberOfElements * sizeof (VkMenuItem *));
    }
  
    if (position == -1){ // place it at the end of the array
	elements[numberOfElements++] = newToggle;
    }
    else{ // place it somewhere in the middle, and move all the others down
	int i;
 
	for (i=numberOfElements-1; i>=position; i--)
	    elements[i+1] = elements[i];
	elements[position] = newToggle;
	numberOfElements++;
    }
    if (instantiated)
	newToggle->_instantiate(baseWidget());
    newToggle->setVisualState(state==-1?FALSE:TRUE);
    return newToggle;
}

VkMenuLabel *
VkMenu::addLabel(const char *name,
		 int position)
{
    VkMenuLabel *newLabel = new VkMenuLabel((char *)name);

    if (numberOfElements == maxNumberOfElements - 1){ // the array is currently full.
	maxNumberOfElements *= 2;
	elements = (VkMenuItem **)realloc(elements, maxNumberOfElements * sizeof (VkMenuItem *));
    }
  
    if (position == -1){ // place it at the end of the array
	elements[numberOfElements++] = newLabel;
    }
    else{ // place it somewhere in the middle, and move all the others down
	int i;
    
	for (i=numberOfElements-1; i>=position; i--)
	    elements[i+1] = elements[i];
	elements[position] = newLabel;
	numberOfElements++;
    }
    if (instantiated)
	newLabel->_instantiate(baseWidget());  
    return newLabel;
}

VkMenuSeparator *
VkMenu::addSeparator(const char *name,
		     int position)
{
    VkMenuSeparator *newSeparator = new VkMenuSeparator((char*)name);

    if (numberOfElements == maxNumberOfElements - 1){ // the array is currently full.
	maxNumberOfElements *= 2;
	elements = (VkMenuItem **)realloc(elements, maxNumberOfElements * sizeof (VkMenuItem *));
    }
  
    if (position == -1){ // place it at the end of the array
	elements[numberOfElements++] = newSeparator;
    }
    else{ // place it somewhere in the middle, and move all the others down
	int i;
    
	for (i=numberOfElements-1; i>=position; i--)
	    elements[i+1] = elements[i];
	elements[position] = newSeparator;
	numberOfElements++;
    }
    if (instantiated)
	newSeparator->_instantiate(baseWidget());  
    return newSeparator;
}

VkSubMenu *
VkMenu::addSubmenu(VkSubMenu *submenu,
		   int position)
{
    if (numberOfElements == maxNumberOfElements - 1){ // the array is currently full.
	maxNumberOfElements *= 2;
	elements = (VkMenuItem **)realloc(elements, maxNumberOfElements * sizeof (VkMenuItem *));
    }
  
    if (position == -1){ // place it at the end of the array
	elements[numberOfElements++] = submenu;
    }
    else{ // place it somewhere in the middle, and move all the others down
	int i;

	for (i=numberOfElements-1; i>=position; i--)
	    elements[i+1] = elements[i];
	elements[position] = submenu;
	numberOfElements++;
    }
    if (instantiated)
	submenu->_instantiate(baseWidget());
    return submenu;
}

VkSubMenu *
VkMenu::addSubmenu(const char *name,
		   int position)
{
    VkSubMenu *newSubMenu = new VkSubMenu(name);
  
    if (numberOfElements == maxNumberOfElements - 1){ // the array is currently full.
	maxNumberOfElements *= 2;
	elements = (VkMenuItem **)realloc(elements, maxNumberOfElements * sizeof (VkMenuItem *));
    }
  
    if (position == -1){ // place it at the end of the array
	elements[numberOfElements++] = newSubMenu;
    }
    else{ // place it somewhere in the middle, and move all the others down
	int i;
    
	for (i=numberOfElements-1; i>=position; i--)
	    elements[i+1] = elements[i];
	elements[position] = newSubMenu;
	numberOfElements++;
    }
    if (instantiated)
	newSubMenu->_instantiate(baseWidget());   
    return newSubMenu;
}


VkSubMenu *
VkMenu::addSubmenu(const char *name,
		   VkMenuDesc *menuDesc,
		   XtPointer defaultClientData,
		   int position)
{
    VkSubMenu *newSubMenu = new VkSubMenu(name);
    VkMenuDesc *current;

    if (numberOfElements == maxNumberOfElements - 1){ // the array is currently full.
	maxNumberOfElements *= 2;
	elements = (VkMenuItem **)realloc(elements, maxNumberOfElements * sizeof (VkMenuItem *));
    }

    if (position == -1){ // place it at the end of the array
	elements[numberOfElements] = newSubMenu;
	numberOfElements++;
    }
    else{ // place it somewhere in the middle, and move all the others down
	int i;

	for (i=numberOfElements-1; i>=position; i--)
	    elements[i+1] = elements[i];
	elements[position] = newSubMenu;
	numberOfElements++;
    }

    for (current = menuDesc;
	 current->menuType != END;
	 current ++){
	switch (current->menuType){
	case ACTION:
	    newSubMenu->addAction(current->name,
				  current->callback,
				  current->clientData ? current->clientData : defaultClientData); 
	    break;
	case CONFIRMFIRSTACTION:
	    newSubMenu->addConfirmFirstAction(current->name,
					      current->callback,
					      current->clientData ? current->clientData : defaultClientData);
	    break;
	case TOGGLE:
	    newSubMenu->addToggle(current->name,
				  current->callback,
				  current->clientData ? current->clientData : defaultClientData);
	    break;
	case LABEL:
	    newSubMenu->addLabel(current->name);
	    break;
	case SEPARATOR:
	    newSubMenu->addSeparator();
	    break;
	case SUBMENU:
	    newSubMenu->addSubmenu(current->name,
				   current->submenu,
				   current->clientData ? current->clientData : defaultClientData);
	    break;
	case RADIOSUBMENU:
	    newSubMenu->addRadioSubmenu(current->name,
					current->submenu,
					current->clientData ? current->clientData : defaultClientData);
	    break;
	}
    }
    if (instantiated)
	newSubMenu->_instantiate(baseWidget());
    return newSubMenu;
}

VkRadioSubMenu *
VkMenu::addRadioSubmenu(VkRadioSubMenu *submenu,
			int position)
{
    if (numberOfElements == maxNumberOfElements - 1){ // the array is currently full.
	maxNumberOfElements *= 2;
	elements = (VkMenuItem **)realloc(elements, maxNumberOfElements * sizeof (VkMenuItem *));
    }
  
    if (position == -1){ // place it at the end of the array
	elements[numberOfElements++] = submenu;
    }
    else{ // place it somewhere in the middle, and move all the others down
	int i;

	for (i=numberOfElements-1; i>=position; i--)
	    elements[i+1] = elements[i];
	elements[position] = submenu;
	numberOfElements++;
    }
    if (instantiated)
	submenu->_instantiate(baseWidget());  
    return submenu;
}

VkRadioSubMenu *
VkMenu::addRadioSubmenu(const char *name,
			int position)
{
    VkRadioSubMenu *newSubMenu = new VkRadioSubMenu(name);
  
    if (numberOfElements == maxNumberOfElements - 1){ // the array is currently full.
	maxNumberOfElements *= 2;
	elements = (VkMenuItem **)realloc(elements, maxNumberOfElements * sizeof (VkMenuItem *));
    }
  
    if (position == -1){ // place it at the end of the array
	elements[numberOfElements++] = newSubMenu;
    }
    else{ // place it somewhere in the middle, and move all the others down
	int i;
    
	for (i=position; i<numberOfElements; i--)
	    elements[i+1] = elements[i];
	elements[position] = newSubMenu;
	numberOfElements++;
    }
    if (instantiated)
	newSubMenu->_instantiate(baseWidget());   
    return newSubMenu;  
}

VkRadioSubMenu *
VkMenu::addRadioSubmenu(const char *name,
			VkMenuDesc *menuDesc,
			XtPointer defaultClientData,
			int position)
{
    VkRadioSubMenu *newSubMenu = new VkRadioSubMenu(name);
    VkMenuDesc *current;

    if (numberOfElements == maxNumberOfElements - 1){ // the array is currently full.
	maxNumberOfElements *= 2;
	elements = (VkMenuItem **)realloc(elements, maxNumberOfElements * sizeof (VkMenuItem *));
    }

    if (position == -1){ // place it at the end of the array
	elements[numberOfElements] = newSubMenu;
	numberOfElements++;
    }
    else{ // place it somewhere in the middle, and move all the others down
	int i;
    
	for (i=numberOfElements-1; i>=position; i--)
	    elements[i+1] = elements[i];
	elements[position] = newSubMenu;
	numberOfElements++;
    }

    for (current = menuDesc;
	 current->menuType != END;
	 current ++){
	switch (current->menuType){
	case ACTION:
	    newSubMenu->addAction(current->name,
				  current->callback,
				  current->clientData ? current->clientData : defaultClientData);
	    break;
	case CONFIRMFIRSTACTION:
	    newSubMenu->addConfirmFirstAction(current->name,
					      current->callback,
					      current->clientData ? current->clientData : defaultClientData);
	    break;
	case TOGGLE:
	    newSubMenu->addToggle(current->name,
				  current->callback,
				  current->clientData ? current->clientData : defaultClientData);
	    break;
	case LABEL:
	    newSubMenu->addLabel(current->name);
	    break;
	case SEPARATOR:
	    newSubMenu->addSeparator();
	    break;
	case SUBMENU:
	    newSubMenu->addSubmenu(current->name,
				   current->submenu,
				   current->clientData ? current->clientData : defaultClientData);
	    break;
	case RADIOSUBMENU:
	    newSubMenu->addRadioSubmenu(current->name,
					current->submenu,
					current->clientData ? current->clientData : defaultClientData);
	    break;
	}
    }
    if (instantiated)
	newSubMenu->_instantiate(baseWidget());
    return newSubMenu;  
}

void
VkMenu::add(VkMenuItem *item,
	    int position)
{
    if (numberOfElements == maxNumberOfElements - 1){ // the array is currently full.
	maxNumberOfElements *= 2;
	elements = (VkMenuItem **)realloc(elements, maxNumberOfElements * sizeof (VkMenuItem *));
    }
  
    if (position == -1){ // place it at the end of the array
	elements[numberOfElements++] = item;
    }
    else{ // place it somewhere in the middle, and move all the others down
	int i;

	for (i=numberOfElements-1; i>=position; i--)
	    elements[i+1] = elements[i];
	elements[position] = item;
	numberOfElements++;
    }
    if (instantiated)
	item->_instantiate(baseWidget());  
}

VkMenuItem *
VkMenu::findNamedItem(const char *name,
		      Boolean caseless)
{
    VkMenuItem *subMenuItem;
    int i;

    for (i=0; i < numberOfElements ; i++){
	if (caseless){
	    if (!strcasecmp(elements[i]->name(), name))
		return elements[i];
	    else if (elements[i]->menuType() == SUBMENU 
		     || elements[i]->menuType() == RADIOSUBMENU){
		subMenuItem = ((VkMenu*)elements[i])->findNamedItem(name,caseless);
		if (subMenuItem)
		    return subMenuItem;
		else
		    continue;
	    }
	}
	else
	    if (elements[i]->name())
		if (!strcmp(elements[i]->name(), name))
		    return elements[i];
		else if (elements[i]->menuType() == SUBMENU 
			 || elements[i]->menuType() == RADIOSUBMENU){
		    subMenuItem = ((VkMenu*)elements[i])->findNamedItem(name,caseless);
		    if (subMenuItem)
			return subMenuItem;
		    else
			continue;
		}
    }
    return NULL;
}
  
VkMenuItem *
VkMenu::activateItem(const char *name)
{
    int i;

    for (i=0; i<numberOfElements; i++)
	if (!strcmp(name, elements[i]->name())){
	    elements[i]->activate();
	    return elements[i];
	}

    return NULL;
}

VkMenuItem *
VkMenu::deactivateItem(const char *name)
{
    int i;

    for (i=0; i<numberOfElements; i++)
	if (!strcmp(name, elements[i]->name())){
	    elements[i]->deactivate();
	    return elements[i];
	}

    return NULL;
}

VkMenuItem *
VkMenu::removeItem(const char *name)
{
    VkMenuItem *result;
    int item_position;
    int i;
  
    item_position = getItemPosition((char*)name);

    result = elements[item_position];

    for (i=item_position+1; i<numberOfElements; i++)
	elements[i-1] = elements[i];

    numberOfElements--;

    result->hide();
    return result;
}

VkMenuItem *
VkMenu::replace(const char */*name*/,
		VkMenuItem */*newItem*/)
{
    return NULL;
}

int
VkMenu::numItems() const
{
    return numberOfElements;
}

int
VkMenu::getItemPosition(VkMenuItem *item)
{
    int i;

    for (i=0; i<numberOfElements; i++)
	if (elements[i] == item)
	    return i;

    return -1;
}

int
VkMenu::getItemPosition(char *name)
{
    int i;

    for (i=0; i<numberOfElements; i++)
	if (!strcmp(elements[i]->name(), name))
	    return i;

    return -1;
}

int
VkMenu::getItemPosition(Widget w)
{
    int i;

    for (i=0; i<numberOfElements; i++)
	if (w == elements[i]->baseWidget())
	    return i;

    return -1;
}

VkMenuItem *
VkMenu::operator[] (int index) const
{
    return elements[index];
}

const char *
VkMenu::className()
{
    return "VkMenu";
}

VkMenu::operator Widget() const
{
    return _baseWidget;
}
