/**
 *
 * VkMenuBar.C
 *
 * This file contains the class implementation for the VkMenuBar class,
 * derived from VkMenu, allowing menu bars in VkWindows.
 *
 * Chris Toshok
 * Copyright (C) 1994
 * The Hungry Programmers, Inc
 *
 **/

#ifndef lint
static char rcsid[] = "$Id: VkMenuBar.C,v 1.2 1995/01/23 10:07:35 toshok Exp $";
#endif /* lint */

#include <Vk/VkMenuBar.h>
#include <Vk/VkHelpPane.h>
#include <iostream.h>

VkMenuBar::VkMenuBar(Boolean showHelpPane) : VkMenu("menuBar")
{
  if (showHelpPane)
    _helpPane = new VkHelpPane;
  else
    _helpPane = NULL;
}

VkMenuBar::VkMenuBar(const char *name,
		     Boolean showHelpPane) : VkMenu(name)
{
  if (showHelpPane)
    _helpPane = new VkHelpPane;
  else
    _helpPane = NULL;  
}

VkMenuBar::VkMenuBar(VkMenuDesc *menuDesc, 
		     XtPointer defaultClientData,
		     Boolean showHelpPane) : VkMenu("menuBar")
{
  VkMenuDesc *current;

  _defaultClientData = defaultClientData;
  current = menuDesc;
  while (current->menuType != END){
    switch (current->menuType){
    case SUBMENU:
      addSubmenu((const char*)current->name,
		 current->submenu,
		 (current->clientData?current->clientData:_defaultClientData));
      break;
    case ACTION:
      break;
    }
    current++;
  }
  if (showHelpPane)
    _helpPane = new VkHelpPane;
  else
    _helpPane = NULL;  
}

VkMenuBar::VkMenuBar(const char *name,
		     VkMenuDesc */*menuDesc*/,
		     XtPointer /*defaultClientData*/,
		     Boolean showHelpPane) : VkMenu(name)
{
  if (showHelpPane)
    _helpPane = new VkHelpPane;
  else
    _helpPane = NULL;  
}

VkMenuBar::~VkMenuBar()
{
  delete _helpPane;
}

void
VkMenuBar::_instantiate(Widget parent)
{
  Arg args[1];
  int i;
  
  _baseWidget = XmCreateMenuBar(parent,
				(char *)name(),
				NULL,0);

  XtManageChild(baseWidget());
  
  for (i=0; i<numberOfElements; i++)
    elements[i]->_instantiate(baseWidget());

  if (_helpPane)
    _helpPane->_instantiate(baseWidget());

  XtSetArg(args[0], XmNmenuHelpWidget, _helpPane->baseWidget());
  XtSetValues(_baseWidget, args, 1);
  
  if (activated == False)
    XtSetSensitive(_baseWidget, False);

  instantiated = TRUE;
}

const char *
VkMenuBar::className()
{
  return "VkMenuBar";
}

VkHelpPane *
VkMenuBar::helpPane() const
{
  return _helpPane;
}

VkMenuItemType
VkMenuBar::menuType()
{
  return BAR;
}

void
VkMenuBar::setLabel(const char */*str*/)
{
  // Does nothing now...
}

Boolean
VkMenuBar::isContainer()
{
  return TRUE;
}

