/**
 *
 * VkMenuToggle.C
 *
 * This file contains the implementation for the VkMenuToggle class,
 * which makes it easier to deal with toggle buttons within a menu interface.
 *
 * Chris Toshok
 * Copyright (C) 1994
 * The Hungry Programmers, Inc
 *
 **/

#ifndef lint
static char rcsid[] = "$Id: VkMenuToggle.C,v 1.2 1995/01/23 10:07:40 toshok Exp $";
#endif /* lint */

#include <Vk/VkMenuToggle.h>

VkMenuToggle::VkMenuToggle(char *name) : VkMenuAction(name)
{
  _initial_state = FALSE;
}

void
VkMenuToggle::setVisualState(Boolean state)
{
  if (instantiated)
    XmToggleButtonGadgetSetState(_baseWidget,
				 state, FALSE);

  _initial_state = state;
}

void
VkMenuToggle::setStateAndNotify(Boolean state)
{
  if (instantiated)
    XmToggleButtonGadgetSetState(_baseWidget,
				 state, TRUE);
  
  _initial_state = state;
}

Boolean
VkMenuToggle::getState()
{
  if (instantiated)
    return XmToggleButtonGadgetGetState(_baseWidget);
  else
    return FALSE;
}

VkMenuItemType
VkMenuToggle::menuType()
{
  return TOGGLE;
}

void
VkMenuToggle::_instantiate(Widget parent)
{
  _baseWidget = XmCreateToggleButtonGadget(parent,
					   (char *)name(),
					   NULL,0);

  if (actionCallback)
    XtAddCallback(_baseWidget, XmNvalueChangedCallback, actionCallback, clientData);
  
  XtManageChild(_baseWidget);
  XmToggleButtonGadgetSetState(_baseWidget,
			       _initial_state, FALSE);
  
  if (activated == False)
    XtSetSensitive(_baseWidget, False);

  instantiated = TRUE;
}

const char *
VkMenuToggle::className()
{
  return "VkMenuToggle";
}

Boolean
VkMenuToggle::isContainer()
{
  return FALSE;
}

void
VkMenuToggle::setLabel(const char *str)
{
  Arg args[1];
  XmString xmstr;

  xmstr = XmStringCreateSimple ((char *)str);
  
  XtSetArg(args[0], XmNlabelString, xmstr);

  XtSetValues(_baseWidget, args, 1);
  XmStringFree(xmstr);  
}
