/**
 *
 * VkNameList.C
 *
 * This file contains the implementation for the VkNameList
 * class, which gives a programmatic interface to a list of
 * strings
 *
 * Chris Toshok
 * Copyright (C) 1995
 * The Hungry Programmers, Inc.
 * All Rights Reserved
 *
 **/

#include <Vk/VkNameList.h>
#include <stdlib.h>
#include <string.h>

VkNameList::VkNameList()
{
    _numberOfElements = 0;
    _maxNumberOfElements = 10;

    _contents = (char**)malloc(sizeof(char*) * _maxNumberOfElements);
}

VkNameList::VkNameList(const VkNameList& givenList)
{
    int i;

    _numberOfElements = givenList._numberOfElements;
    _maxNumberOfElements = givenList._maxNumberOfElements;

    _contents = (char**)malloc(sizeof(char*) * _maxNumberOfElements);

    for (i=0; i<_numberOfElements; i++)
	_contents[i] = strdup(givenList._contents[i]);
}

VkNameList::VkNameList(char *name)
{
    _numberOfElements = 0;
    _maxNumberOfElements = 10;

    _contents = (char**)malloc(sizeof(char*) * _maxNumberOfElements);

    _contents[_numberOfElements++] = strdup(name);
}

VkNameList::~VkNameList()
{
    int i;
    
    for (i=0; i<_numberOfElements; i++)
	free(_contents[i]);
    
    free(_contents);
}

void
VkNameList::add(char *name)
{
    _contents[_numberOfElements++] = strdup(name);
}

void
VkNameList::add(const VkNameList &givenList)
{
    int i;

    for (i=0; i<givenList._numberOfElements; i++)
	add(givenList._contents[i]);
}

void
VkNameList::remove(char *name)
{
    int i,j;

    for (i=0; i<_numberOfElements; i++)
    {
	if (!strcmp(name, _contents[i]))
	{
	    free(_contents[i]);
	    for (j=i; j<_numberOfElements-1; j++)
		_contents[j] = _contents[j+1];
	    _numberOfElements--;
	}
    }
}

VkNameList& 
VkNameList::operator=(const VkNameList &givenList)
{
    int i;

    for (i=0; i<_numberOfElements; i++)
	free(_contents[i]);

    if (givenList._numberOfElements > _maxNumberOfElements)
	_maxNumberOfElements = givenList._numberOfElements + 10;
    
    _contents = (char**)realloc(_contents, sizeof(char *) * _maxNumberOfElements);

    _numberOfElements = givenList._numberOfElements;

    for (i=0; i<_numberOfElements; i++)
	_contents[i] = strdup(givenList._contents[i]);

    return *this;  
}

void
VkNameList::sort()
{
    qsort((void*)_contents, _numberOfElements, sizeof(char*), (int(*)(const void*, const void*))strcmp);
}

void
VkNameList::reverse()
{
    int i;
    char *temp;

    // in the case of an even number of elements, the following does
    // one more swap than necessary, but that's really A-OK.

    for (i=0; i<=_numberOfElements/2; i++)
    {
	temp = _contents[i];
	_contents[i] = _contents[_numberOfElements-i];
	_contents[_numberOfElements-i] = temp;
    }
}

void
VkNameList::removeDuplicates()
{
    int i,j,k;

    for (i=0; i<_numberOfElements-1; i++)
	for (j=i; j<_numberOfElements; j++)
	    if (!strcmp(_contents[i], _contents[j]))
	    {
		free (_contents[j]);

		for (k=j; k<_numberOfElements-1; k++)
		    _contents[k] = _contents[k+1];

		_numberOfElements--;
	    }
}

int 
VkNameList::size()
{
    return _numberOfElements;
}

int
VkNameList::exists(char *name)
{
    int i;

    for (i=0; i<_numberOfElements; i++)
	if (!strcmp(name, _contents[i]))
	    return 1;

    return 0;
}

char *
VkNameList::operator[](int index)
{
    return _contents[index];
}

int
VkNameList::operator==(const VkNameList& givenList)
{
    int i;

    if (_numberOfElements != givenList._numberOfElements)
	return 0;

    for (i=0; i<_numberOfElements; i++)
	if (strcmp(_contents[i], givenList._contents[i]))
	    return 0;

    return 1;
}

char *
VkNameList::mostCommonString()
{
    return NULL; // FIX ME
}

VkNameList *
VkNameList::completeName(char *name,
			 char * &completedName,
			 int &nMatching)
{
    return NULL; // FIX ME
}

VkNameList::operator char **()
{
    static char **value = NULL;
    int i;

    value = (char**)XtRealloc((char*)value, sizeof(char*) * (_numberOfElements + 1));

    
    for (i=0; i<_numberOfElements; i++)
	value[i] = strdup(_contents[i]);

    value[_numberOfElements] = NULL;
    
    return value;
}

VkNameList::operator XmStringTable()
{
    return NULL;
}


