/**
 *
 * VkOptionMenu.C
 *
 * This file contains the implementation for the VkOptionMenu class.
 * This class makes it easier to deal with option menus.  However, since
 * I haven't actually played around with the real ViewKit, I have no idea
 * what they look like.  ho hum :)
 *
 * Chris Toshok
 * Copyright (C) 1994
 * The Hungry Programmers, Inc
 *
 **/

#ifndef lint
static char rcsid[] = "$Id: VkOptionMenu.C,v 1.2 1995/01/23 10:07:41 toshok Exp $";
#endif /* lint */

#include <Vk/VkOptionMenu.h>

VkOptionMenu::VkOptionMenu(Widget parent,
			   VkMenuDesc *menuDesc,
			   XtPointer defaultClientData) : VkMenu ("optionMenu")
{
  VkMenuDesc *current;
  
  for (current = menuDesc;
       current->menuType != END;
       current ++)
    switch (current->menuType){
    case ACTION:
      addAction(current->name,
		current->callback,
		current->clientData==NULL ? defaultClientData : current->clientData);
      break;
    case CONFIRMFIRSTACTION:
      addConfirmFirstAction(current->name,
			    current->callback,
			    current->clientData==NULL ? defaultClientData : current->clientData);
      break;
    case TOGGLE:
      addToggle(current->name,
		current->callback,
		current->clientData==NULL ? defaultClientData : current->clientData);
      break;
    case LABEL:
      addLabel(current->name);
      break;
    case SEPARATOR:
      addSeparator();
      break;
    case SUBMENU:
      addSubmenu(current->name,
		 current->submenu,
		 defaultClientData);
      break;
    case RADIOSUBMENU:
      addRadioSubmenu(current->name,
		      current->submenu,
		      defaultClientData);
      break;
    }

  _instantiate(parent);
}

VkOptionMenu::VkOptionMenu(Widget parent,
			   const char *name,
			   VkMenuDesc *menuDesc,
			   XtPointer defaultClientData) : VkMenu (name)
{
  VkMenuDesc *current;

  if (menuDesc)
    for (current = menuDesc;
	 current->menuType != END;
	 current ++)
      switch (current->menuType){
      case ACTION:
	addAction(current->name,
		  current->callback,
		  current->clientData==NULL ? defaultClientData : current->clientData);
	break;
      case CONFIRMFIRSTACTION:
	addConfirmFirstAction(current->name,
			      current->callback,
			      current->clientData==NULL ? defaultClientData : current->clientData);
	break;
      case TOGGLE:
	addToggle(current->name,
		  current->callback,
		  current->clientData==NULL ? defaultClientData : current->clientData);
	break;
      case LABEL:
	addLabel(current->name);
	break;
      case SEPARATOR:
	addSeparator();
	break;
      case SUBMENU:
	addSubmenu(current->name,
		   current->submenu,
		   defaultClientData);
	break;
      case RADIOSUBMENU:
	addRadioSubmenu(current->name,
			current->submenu,
			defaultClientData);
	break;
      }
  _instantiate(parent);
}

void
VkOptionMenu::set(char *name)
{
  VkMenuItem *result;
  Arg args[1];

  result = findNamedItem(name);

  XtSetArg(args[0], XmNmenuHistory, result->baseWidget());

  XtSetValues(_baseWidget, args, 1);
}

void
VkOptionMenu::set(int index)
{
  Arg args[1];

  XtSetArg(args[0], XmNmenuHistory, elements[index]->baseWidget());

  XtSetValues(_baseWidget, args, 1);
}

void
VkOptionMenu::set(VkMenuItem *item)
{
  Arg args[1];

  XtSetArg(args[0], XmNmenuHistory, item->baseWidget());

  XtSetValues(_baseWidget, args, 1);
}

int
VkOptionMenu::getIndex()
{
  Arg args[1];
  Widget widgetId;

  XtSetArg (args[0], XmNmenuHistory, &widgetId);

  XtGetValues(_baseWidget, args, 1);

  return getItemPosition(widgetId);
}

VkMenuItem *
VkOptionMenu::getItem()
{
  Arg args[1];
  Widget widgetId;
  
  XtSetArg (args[0], XmNmenuHistory, &widgetId);

  XtGetValues(_baseWidget, args, 1);

  return elements[getItemPosition(widgetId)];
}

void
VkOptionMenu::forceWidth(int /*width*/)
{
}

VkMenuItemType
VkOptionMenu::menuType()
{
  return OPTION;
}

const char *
VkOptionMenu::className()
{
  return "VkOptionMenu";
}

Boolean
VkOptionMenu::isContainer()
{
  return TRUE;
}

void
VkOptionMenu::setLabel(const char *str)
{
  Widget label = XmOptionLabelGadget(_baseWidget);
  XmString xmstr;
  Arg args[1];
 
  xmstr = XmStringCreateSimple((char*)str);

  XtSetArg(args[0], XmNlabelString, xmstr);

  XtSetValues(label, args, 1);  

  XmStringFree(xmstr);
}

void
VkOptionMenu::_instantiate(Widget parent)
{
  Arg args[10];
  XmString xmstr;
  int i;
  char buf[100];

  sprintf (buf, "%s_pulldown", (char*)name());
  
  _pulldown = XmCreatePulldownMenu(parent,
				   buf,
				   NULL,0);

  xmstr = XmStringCreateSimple("");
  XtSetArg(args[0], XmNsubMenuId, _pulldown);
  XtSetArg(args[1], XmNlabelString, xmstr);
  
  _baseWidget = XmCreateOptionMenu(parent,
				   (char*)name(),
				   args, 2);

  for (i=0; i<numberOfElements; i++)
    elements[i]->_instantiate(_pulldown);

  XtManageChild(_baseWidget);
 
  XmStringFree(xmstr);
 
  instantiated = TRUE;
}

Widget
VkOptionMenu::pulldown()
{
  return _pulldown;
}
