/**
 *
 * VkPipe.C
 *
 * This file contains the implementation for the VkPipe
 * class, which gives the programmer an interface to 
 * unix pipes.
 *
 * Chris Toshok
 * Copyright (C) 1995
 * The Hungry Programmers, Inc.
 * All Rights Reserved 
 *
 **/

#include <Vk/VkPipe.h>
#include <Vk/VkFatalErrorDialog.h>
#include <fcntl.h>
#include <sys/file.h>  // dumb HPUX
#include <unistd.h>

VkPipe::VkPipe(int &fd)
{
    if (pipe(pipefd) < 0)
	theFatalErrorDialog->postModal("pipe() system call failed");

    fd = pipefd[1];

    if (fcntl(pipefd[0], F_SETFL, FNDELAY) < 0)
	theFatalErrorDialog->postModal("fcntl() on read fd failed");

    if (fcntl(pipefd[1], F_SETFL, FNDELAY) < 0)
	theFatalErrorDialog->postModal("fcntl() on write fd failed");

    _input = new VkInput();
    _input->attach(pipefd[0], XtInputReadMask);
    _input->addCallback(VkInput::inputCallback, 
			this,
			(VkCallbackMethod)&(VkPipe::handleInput));;
}

VkPipe::~VkPipe()
{
    close(pipefd[0]);
    close(pipefd[1]);
}

int
VkPipe::operator[](End whichEnd)
{
    switch (whichEnd){
    case READ:
	return pipefd[0];
	break;
    case WRITE:
	return pipefd[1];
	break;
    }
}

void
VkPipe::handleInput(VkCallbackObject *obj, void *, void *callData)
{
    callCallbacks(VkPipe::inputCallback, this);
}

const char *const 
VkPipe::inputCallback = "VkPipe::inputCallback";
