/**
 *
 * VkPopupMenu.C
 *
 * This file contains the source for the class implementation of the
 * VkPopupMenu class.  This class makes doing popup menus a lot easier.
 *
 * Chris Toshok
 * Copyright (C) 1994
 * The Hungry Programmers, Inc
 *
 **/

#ifndef lint
static char rcsid[] = "$Id: VkPopupMenu.C,v 1.3 1995/02/15 19:11:51 toshok Exp $";
#endif /* lint */

#include <Vk/VkPopupMenu.h>

VkPopupMenu::VkPopupMenu(VkMenuDesc *menuDesc,
			 XtPointer defaultClientData) : VkMenu("PopupMenu")
{
  VkMenuDesc *current;

  for (current = menuDesc;
       current->menuType != END;
       current ++)
    switch (current->menuType){
    case ACTION:
      addAction(current->name,
		current->callback,
		current->clientData==NULL ? defaultClientData : current->clientData);
      break;
    case CONFIRMFIRSTACTION:
      addConfirmFirstAction(current->name,
			    current->callback,
			    current->clientData==NULL ? defaultClientData : current->clientData);
      break;
    case TOGGLE:
      addToggle(current->name,
		current->callback,
		current->clientData==NULL ? defaultClientData : current->clientData);
      break;
    case LABEL:
      addLabel(current->name);
      break;
    case SEPARATOR:
      addSeparator();
      break;
    case SUBMENU:
      addSubmenu(current->name,
		 current->submenu,
		 defaultClientData);
      break;
    case RADIOSUBMENU:
      addRadioSubmenu(current->name,
		      current->submenu,
		      defaultClientData);
      break;
    }
  

}

VkPopupMenu::VkPopupMenu(const char *name,
			 VkMenuDesc *menuDesc,
			 XtPointer defaultClientData) : VkMenu(name)
{
  VkMenuDesc *current;

  for (current = menuDesc;
       current->menuType != END;
       current ++)
    switch (current->menuType){
    case ACTION:
      addAction(current->name,
		current->callback,
		current->clientData==NULL ? defaultClientData : current->clientData);
      break;
    case CONFIRMFIRSTACTION:
      addConfirmFirstAction(current->name,
			    current->callback,
			    current->clientData==NULL ? defaultClientData : current->clientData);
      break;
    case TOGGLE:
      addToggle(current->name,
		current->callback,
		current->clientData==NULL ? defaultClientData : current->clientData);
      break;
    case LABEL:
      addLabel(current->name);
      break;
    case SEPARATOR:
      addSeparator();
      break;
    case SUBMENU:
      addSubmenu(current->name,
		 current->submenu,
		 defaultClientData);
      break;
    case RADIOSUBMENU:
      addRadioSubmenu(current->name,
		      current->submenu,
		      defaultClientData);
      break;
    }
  
}

VkPopupMenu::VkPopupMenu(Widget parent,
			 VkMenuDesc *menuDesc,
			 XtPointer defaultClientData) : VkMenu ("PopupMenu")
{
  VkMenuDesc *current;

  for (current = menuDesc;
       current->menuType != END;
       current ++)
    switch (current->menuType){
    case ACTION:
      addAction(current->name,
		current->callback,
		current->clientData==NULL ? defaultClientData : current->clientData);
      break;
    case CONFIRMFIRSTACTION:
      addConfirmFirstAction(current->name,
			    current->callback,
			    current->clientData==NULL ? defaultClientData : current->clientData);
      break;
    case TOGGLE:
      addToggle(current->name,
		current->callback,
		current->clientData==NULL ? defaultClientData : current->clientData);
      break;
    case LABEL:
      addLabel(current->name);
      break;
    case SEPARATOR:
      addSeparator();
      break;
    case SUBMENU:
      addSubmenu(current->name,
		 current->submenu,
		 defaultClientData);
      break;
    case RADIOSUBMENU:
      addRadioSubmenu(current->name,
		      current->submenu,
		      defaultClientData);
      break;
    }
  

  attach (parent);
}

VkPopupMenu::VkPopupMenu(Widget parent,
			 const char *name,
			 VkMenuDesc *menuDesc,
			 XtPointer defaultClientData) : VkMenu (name)
{
  VkMenuDesc *current;

  for (current = menuDesc;
       current->menuType != END;
       current ++)
    switch (current->menuType){
    case ACTION:
      addAction(current->name,
		current->callback,
		current->clientData==NULL ? defaultClientData : current->clientData);
      break;
    case CONFIRMFIRSTACTION:
      addConfirmFirstAction(current->name,
			    current->callback,
			    current->clientData==NULL ? defaultClientData : current->clientData);
      break;
    case TOGGLE:
      addToggle(current->name,
		current->callback,
		current->clientData==NULL ? defaultClientData : current->clientData);
      break;
    case LABEL:
      addLabel(current->name);
      break;
    case SEPARATOR:
      addSeparator();
      break;
    case SUBMENU:
      addSubmenu(current->name,
		 current->submenu,
		 defaultClientData);
      break;
    case RADIOSUBMENU:
      addRadioSubmenu(current->name,
		      current->submenu,
		      defaultClientData);
      break;
    }
  
  attach (parent);  
}

void
VkPopupMenu::attach(Widget w)
{
  if (!instantiated){
    _baseWidget = XmCreatePopupMenu(w,
				    (char *)name(),
				    NULL,0);
    build(baseWidget());
  }
  
  XtAddEventHandler(w,
		    ButtonPressMask,
		    FALSE,
		    VkPopupMenu::_popup,
		    (XtPointer) this);
}

void
VkPopupMenu::build(Widget parent)
{
  int i;
  
  for (i=0; i< numberOfElements; i++)
    elements[i]->_instantiate(parent);
  instantiated = TRUE;
}

void
VkPopupMenu::show()
{
    VkMenuItem::show();
}

void
VkPopupMenu::show(XEvent *buttonPressEvent)
{
}

VkMenuItemType
VkPopupMenu::menuType()
{
  return POPUP;
}

const char *
VkPopupMenu::className()
{
  return "VkPopupMenu";
}

void
VkPopupMenu::setLabel(const char *)
{
}

void
VkPopupMenu::_instantiate(Widget parent)
{
}

Boolean
VkPopupMenu::isContainer()
{
  return TRUE;
}

void
VkPopupMenu::_popup(Widget w,
		    XtPointer clientData,
		    XEvent *event,
		    Boolean *)
{
  XButtonPressedEvent *e = (XButtonPressedEvent *)event;

  if (e->button == Button3){
    XmMenuPosition(((VkPopupMenu*)clientData)->baseWidget(),
		   e);
    XtManageChild(((VkPopupMenu*)clientData)->baseWidget());
  }
}

