/**
 *
 * VkRadioGroup.C
 *
 * This file contains the class implementation for
 * the VkRadioGroup class, which allows programmers
 * to enforce radio-like behavior on toggle buttons
 * and pushbuttons which inhabit different parts
 * of the interface (that is, are not children of the
 * same RowColumn).
 *
 * Copyright (C) 1994
 * The Hungry Programmers, Inc.
 * All Rights Reserved
 *
 **/

#include <Vk/VkRadioGroup.h>
#include <Xm/ToggleB.h>
#include <Xm/ToggleBG.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>

VkRadioGroup::VkRadioGroup() : VkWidgetList()
{
    currentlySelected = -1;
}

void 
VkRadioGroup::add(Widget item)
{
    if (XmIsToggleButton(item) || XmIsToggleButtonGadget(item)){
	Arg args[1];
	
	XtAddCallback(item, XmNvalueChangedCallback, changeAppearanceCallback, this);

	VkWidgetList::add(item);

	XtSetArg(args[0], XmNindicatorType, XmONE_OF_MANY);
	XtSetValues(item, args, 1);
    }
    else if (XmIsPushButton(item) || XmIsPushButtonGadget(item)){
	XtAddCallback(item, XmNactivateCallback, changeAppearanceCallback, this);
	VkWidgetList::add(item);
    }
    else
	fprintf (stderr, "Tried to add non (Push|Toggle)Button to RadioGroup\n");
}

void 
VkRadioGroup::add(VkComponent *component)
{
    add(component->baseWidget());
}

void 
VkRadioGroup::remove(Widget item)
{
    if (currentlySelected == indexOfItem(item))
        currentlySelected = -1;
    else if (currentlySelected > indexOfItem(item))
        currentlySelected --;

    if (XmIsToggleButton(item) || XmIsToggleButtonGadget(item)){
	XtRemoveCallback(item, XmNvalueChangedCallback, changeAppearanceCallback, this);
    }
    else if (XmIsPushButton(item) || XmIsPushButtonGadget(item)){
	XtRemoveCallback(item, XmNactivateCallback, changeAppearanceCallback, this);
    }

    VkWidgetList::remove(item);
}

void 
VkRadioGroup::remove(VkComponent *component)
{
    remove(component->baseWidget());
}

void 
VkRadioGroup::removeFirst()
{
    remove((*this)[0]);
}

void 
VkRadioGroup::removeLast()
{
    remove((*this)[size() - 1]);
}

void
VkRadioGroup::changeAppearance(int selection)
{
    Pixel armColor;
    Pixel temp;
    Pixmap armPixmap;
    int i;

    for (i=0; i < size(); i++)
	if (XmIsToggleButton((*this)[i]))
	    XmToggleButtonSetState((*this)[i],
				   (selection == i), False);
	else if (XmIsToggleButtonGadget((*this)[i]))
	    XmToggleButtonGadgetSetState((*this)[i],
					 (selection == i), False);
	else if (XmIsPushButton((*this)[i]) || XmIsPushButtonGadget((*this)[i])){
	    if ((i = currentlySelected || i == selection) && 
		currentlySelected != selection ) {
		XtVaGetValues((*this)[i], XmNarmColor, &armColor,
			      XtNbackground, &temp,
			      NULL);
		XtVaSetValues((*this)[i], XtNbackground, armColor,
			      XmNarmColor, temp,
			      NULL);
	    }
	}

    currentlySelected = selection;
}

void
VkRadioGroup::changeAppearanceCallback(Widget w, 
                                       XtPointer clientData, 
                                       XtPointer callData)
{
    VkRadioGroup *obj = (VkRadioGroup*)clientData;

    obj->changeAppearance(obj->indexOfItem(w));
}
