/**
 *
 * VkRepeatButton.C
 *
 * This file contains the class implementation for
 * the VkRepeatButton class, which allows programmers
 * to add buttons that repeatedly call callbacks once
 * armed.
 *
 * Copyright (C) 1994
 * The Hungry Programmers, Inc.
 * All Rights Reserved
 *
 **/

#include <Vk/VkApp.h>
#include <Vk/VkRepeatButton.h>
#include <Vk/VkResource.h>

String
VkRepeatButton::_defaultResources[] = {
"*initialDelay: 1000",
"*repeatDelay: 200",
};

VkRepeatButton::VkRepeatButton(char *name, 
                               Widget parent, 
                               VkRepeatButtonType type) : VkComponent(name)
{
  setDefaultResources(parent, _defaultResources);


  _type = type;

  switch (_type){
  case RB_pushButton:
        _baseWidget = XmCreatePushButton(parent, name, NULL, 0);
        break;
  case RB_pushButtonGadget:
        _baseWidget = XmCreatePushButtonGadget(parent, name, NULL, 0);
        break;
  case RB_arrowButton:
        _baseWidget = XmCreateArrowButton(parent, name, NULL, 0);
        break;
  case RB_arrowButtonGadget:
        _baseWidget = XmCreateArrowButtonGadget(parent, name, NULL, 0);
        break;
  }

  installDestroyHandler();

  _initialDelay = (long) VkGetResource(_baseWidget,
                                       "initialDelay", "InitialDelay",
                                       XmRString, "1000");
  _repeatDelay = (long) VkGetResource(_baseWidget,
                                      "repeatDelay", "RepeatDelay",
                                      XmRString, "200");
}                        
 
void 
VkRepeatButton::setParameters(long initial, 
                              long repeat)
{
  _initialDelay = initial;
  _repeatDelay = repeat;
}
  
VkRepeatButtonType 
VkRepeatButton::type()
{
  return _type;
}

void
VkRepeatButton::_beginRepeat(XtPointer callData)
{
  XmAnyCallbackStruct *cb = (XmAnyCallbackStruct*)callData;

  myCallbackInfo.reason = cb->reason;
  myCallbackInfo.event = cb->event;

  _repeatId = XtAppAddTimeOut(theApplication->appContext(),
   	                     (unsigned long)_initialDelay, _repeatTimeOut, this);
}

void
VkRepeatButton::_beginRepeatCallback(Widget w,
                                     XtPointer clientData, 
                                     XtPointer callData)
{
  VkRepeatButton *obj = (VkRepeatButton*)clientData;        
  
  obj->_beginRepeat(callData);
}

void
VkRepeatButton::_endRepeat()
{
  XtRemoveTimeOut(_repeatId);
}

void
VkRepeatButton::_endRepeatCallback(Widget w,
                                   XtPointer clientData, 
                                   XtPointer callData)
{
  VkRepeatButton *obj = (VkRepeatButton*)clientData;

  obj->_endRepeat();
}

void
VkRepeatButton::_repeat()
{

  _repeatId = XtAppAddTimeOut(theApplication->appContext(),  
          	              _repeatDelay,
                              _repeatTimeOut, this);

  callCallbacks(VkRepeatButton::buttonCallback,
                (void*) &myCallbackInfo);
}

void
VkRepeatButton::_repeatTimeOut(XtPointer clientData,
                               XtIntervalId *id)
{                              
  VkRepeatButton *obj = (VkRepeatButton*)clientData;

  obj->_repeat();
}

const char *
VkRepeatButton::className()
{
  return "VkRepeatButton";
}

const char *const
VkRepeatButton::buttonCallback = "VkRepeatButton::buttonCallback";

