/**
 *
 * VkSubMenu.C
 *
 * This file contains the source code for the implementation of the VkSubMenu
 * Class, which allows the programmer to nest menus inside of one another.
 *
 * Chris Toshok
 * Copyright (C) 1994
 * The Hungry Programmers, Inc
 *
 **/

#ifndef lint
static char rcsid[] = "$Id: VkSubMenu.C,v 1.2 1995/01/23 10:07:59 toshok Exp $";
#endif /* lint */

#include <Vk/VkSubMenu.h>
#include <iostream.h>


VkSubMenu::VkSubMenu(const char *name,
		     VkMenuDesc */*menuDesc*/,
		     XtPointer /*defaultClientData*/) : VkMenu(name)
{
  _tearOff = FALSE;
  
}

void
VkSubMenu::showTearOff(Boolean showIt)
{
  Arg args[1];
  
  if (instantiated){
    if (showIt == TRUE)
      XtSetArg(args[0], XmNtearOffModel, XmTEAR_OFF_ENABLED);
    else
      XtSetArg(args[0], XmNtearOffModel, XmTEAR_OFF_DISABLED);
  
    XtSetValues(_pulldown, args, 1);
  }

  _tearOff = showIt;
}

Widget
VkSubMenu::pulldown()
{
  return _pulldown;
}

VkMenuItemType
VkSubMenu::menuType()
{
  return SUBMENU;
}

Boolean
VkSubMenu::isContainer()
{
  return TRUE;
}

void
VkSubMenu::setLabel(const char *str)
{
  Arg args[1];
  XmString xmstr;

  xmstr = XmStringCreateSimple ((char *)str);
  
  XtSetArg(args[0], XmNlabelString, xmstr);

  XtSetValues(_baseWidget, args,1);
  XmStringFree(xmstr);  
}

void
VkSubMenu::_instantiate(Widget parent)
{
  Arg args[1];
  int i;

  if (_tearOff)
    XtSetArg(args[0], XmNtearOffModel, XmTEAR_OFF_ENABLED);
  else
    XtSetArg(args[0], XmNtearOffModel, XmTEAR_OFF_DISABLED);
    
  _pulldown = XmCreatePulldownMenu(parent,
				   (char *)name(),
				   args,1);

  XtSetArg(args[0], XmNsubMenuId, _pulldown);
  
  _baseWidget = XmCreateCascadeButtonGadget(parent,
					    (char *)name(),
					    args,1);

  XtManageChild(baseWidget());
  
  for (i=0; i<numberOfElements; i++)
    elements[i]->_instantiate(_pulldown);

  if (activated == False)
    XtSetSensitive(_baseWidget, False);

  instantiated = TRUE;
}

const char *
VkSubMenu::className()
{
  return "VkSubMenu";
}

VkSubMenu::operator Widget() const
{
  return _baseWidget;
}
