/**
 *
 * VkWidgetList.C
 *
 * This file contains the class implementation for
 * the VkWidgetList class, which contains a dynamically
 * resizable array of widgets.
 *
 * Copyright (C) 1994
 * The Hungry Programmers, Inc.
 * All Rights Reserved
 *
 **/
  
#include <Vk/VkWidgetList.h>
#include <stdlib.h>

VkWidgetList::VkWidgetList()
{
    _numberOfItems = 0;
    _maxNumberOfItems = 10;

    _items = (Widget*)malloc(_maxNumberOfItems * sizeof(Widget));
}
  
void
VkWidgetList::doReallocIfNecessary()
{
    if (_numberOfItems == _maxNumberOfItems - 1){
	_maxNumberOfItems *= 2;

	_items = (Widget*)realloc(_items, _maxNumberOfItems * sizeof (Widget));
    }
}

void 
VkWidgetList::add(Widget newItem)
{
    _items[_numberOfItems++] = newItem;  
  
    doReallocIfNecessary();  
}

void
VkWidgetList::add(VkComponent *component)
{
    add(component->baseWidget());
}


void 
VkWidgetList::remove(Widget item)
{
    int i,j;

    for (i=0; i < _numberOfItems; i++)
	if (_items[i] == item)
	    break;

    for (j=i; j < _numberOfItems-1; j++)
        _items[j] = _items[j+1];

    _numberOfItems--;
}

void
VkWidgetList::remove(VkComponent *component)
{
    remove(component->baseWidget());
}

void
VkWidgetList::removeLast()
{
    _numberOfItems--;
}

void
VkWidgetList::removeFirst()
{
    int i;

    for (i=0; i < _numberOfItems -1; i++)
	_items[i] = _items[i+1];

    _numberOfItems--;
}

int 
VkWidgetList::indexOfItem(Widget item)
{
    int i;

    for (i=0; i < _numberOfItems; i++)
        if (_items[i] == item)
	    return i;

    return -1;
}

Widget 
VkWidgetList::operator[](int index) const
{
    return _items[index];
}
