/**
 *
 * VkWindow.C
 *
 * This file contains the implementation for the VkWindow class,
 * which forms a derived class from VkSimpleWindow, adding menubar
 * functionality.
 *
 * Chris Toshok
 * Copyright (C) 1994
 * The Hungry Programmers, Inc
 *
 **/

#ifndef lint
static char rcsid[] = "$Id: VkWindow.C,v 1.3 1995/02/21 00:21:39 toshok Exp $";
#endif /* lint */

#include <Vk/VkWindow.h>
#include <iostream.h>

VkWindow::VkWindow(const char *name,
		   ArgList args,
		   Cardinal argCount) : VkSimpleWindow(name, args, argCount)
{
    _menuBar = NULL;
    _menuBarInstantiated = FALSE;
}

VkWindow::~VkWindow()
{
    delete _menuBar;
}

const char *
className()
{
    return "VkWindow";
}

void
VkWindow::show()
{
    if (!_menuBarInstantiated && _menuBar)
	_menuBar->_instantiate(mainWindowWidget());
    
    VkSimpleWindow::show();
}

const char *
VkWindow::className()
{
    return "VkWindow";
}

void
VkWindow::setMenuBar(VkMenuBar *menuObj)
{
    _menuBar = menuObj;

}

void
VkWindow::setMenuBar(VkMenuDesc *menuDesc)
{
    VkMenuDesc *current = menuDesc;

    if (_menuBar == NULL)
	_menuBar = new VkMenuBar("Application");
  
    while (current->menuType != END){
	switch (current->menuType){
	case SUBMENU:
	    _menuBar->addSubmenu((const char*)current->name,
				 current->submenu,
				 current->clientData);
	    break;
	case ACTION:
	    break;
	}
	current++;
    }
}

VkMenuBar *
VkWindow::menu() const
{
    return _menuBar;
}

VkSubMenu *
VkWindow::addMenuPane(const char *name)
{
    if (_menuBar == NULL) // Create a new one
	_menuBar = new VkMenuBar();


    return (_menuBar->addSubmenu(name));
}

VkSubMenu *
VkWindow::addMenuPane(const char *name,
		      VkMenuDesc *menudesc)
{

    if (!_menuBar) // Create a new one
	_menuBar = new VkMenuBar();
  
    return (_menuBar->addSubmenu(name,
				 menudesc,
				 NULL));
}
